# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/datafusion.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16proto/datafusion.proto\x12\ndatafusion\"\"\n\x0e\x43olumnRelation\x12\x10\n\x08relation\x18\x01 \x01(\t\"D\n\x06\x43olumn\x12\x0c\n\x04name\x18\x01 \x01(\t\x12,\n\x08relation\x18\x02 \x01(\x0b\x32\x1a.datafusion.ColumnRelation\"Z\n\x07\x44\x66\x46ield\x12 \n\x05\x66ield\x18\x01 \x01(\x0b\x32\x11.datafusion.Field\x12-\n\tqualifier\x18\x02 \x01(\x0b\x32\x1a.datafusion.ColumnRelation\"\x97\x01\n\x08\x44\x66Schema\x12$\n\x07\x63olumns\x18\x01 \x03(\x0b\x32\x13.datafusion.DfField\x12\x34\n\x08metadata\x18\x02 \x03(\x0b\x32\".datafusion.DfSchema.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xb3\n\n\x0fLogicalPlanNode\x12\x38\n\x0clisting_scan\x18\x01 \x01(\x0b\x32 .datafusion.ListingTableScanNodeH\x00\x12\x30\n\nprojection\x18\x03 \x01(\x0b\x32\x1a.datafusion.ProjectionNodeH\x00\x12.\n\tselection\x18\x04 \x01(\x0b\x32\x19.datafusion.SelectionNodeH\x00\x12&\n\x05limit\x18\x05 \x01(\x0b\x32\x15.datafusion.LimitNodeH\x00\x12.\n\taggregate\x18\x06 \x01(\x0b\x32\x19.datafusion.AggregateNodeH\x00\x12$\n\x04join\x18\x07 \x01(\x0b\x32\x14.datafusion.JoinNodeH\x00\x12$\n\x04sort\x18\x08 \x01(\x0b\x32\x14.datafusion.SortNodeH\x00\x12\x32\n\x0brepartition\x18\t \x01(\x0b\x32\x1b.datafusion.RepartitionNodeH\x00\x12\x37\n\x0e\x65mpty_relation\x18\n \x01(\x0b\x32\x1d.datafusion.EmptyRelationNodeH\x00\x12\x44\n\x15\x63reate_external_table\x18\x0b \x01(\x0b\x32#.datafusion.CreateExternalTableNodeH\x00\x12*\n\x07\x65xplain\x18\x0c \x01(\x0b\x32\x17.datafusion.ExplainNodeH\x00\x12(\n\x06window\x18\r \x01(\x0b\x32\x16.datafusion.WindowNodeH\x00\x12*\n\x07\x61nalyze\x18\x0e \x01(\x0b\x32\x17.datafusion.AnalyzeNodeH\x00\x12/\n\ncross_join\x18\x0f \x01(\x0b\x32\x19.datafusion.CrossJoinNodeH\x00\x12(\n\x06values\x18\x10 \x01(\x0b\x32\x16.datafusion.ValuesNodeH\x00\x12\x35\n\textension\x18\x11 \x01(\x0b\x32 .datafusion.LogicalExtensionNodeH\x00\x12\x44\n\x15\x63reate_catalog_schema\x18\x12 \x01(\x0b\x32#.datafusion.CreateCatalogSchemaNodeH\x00\x12&\n\x05union\x18\x13 \x01(\x0b\x32\x15.datafusion.UnionNodeH\x00\x12\x37\n\x0e\x63reate_catalog\x18\x14 \x01(\x0b\x32\x1d.datafusion.CreateCatalogNodeH\x00\x12\x37\n\x0esubquery_alias\x18\x15 \x01(\x0b\x32\x1d.datafusion.SubqueryAliasNodeH\x00\x12\x31\n\x0b\x63reate_view\x18\x16 \x01(\x0b\x32\x1a.datafusion.CreateViewNodeH\x00\x12,\n\x08\x64istinct\x18\x17 \x01(\x0b\x32\x18.datafusion.DistinctNodeH\x00\x12\x32\n\tview_scan\x18\x18 \x01(\x0b\x32\x1d.datafusion.ViewTableScanNodeH\x00\x12\x36\n\x0b\x63ustom_scan\x18\x19 \x01(\x0b\x32\x1f.datafusion.CustomTableScanNodeH\x00\x12*\n\x07prepare\x18\x1a \x01(\x0b\x32\x17.datafusion.PrepareNodeH\x00\x12-\n\tdrop_view\x18\x1b \x01(\x0b\x32\x18.datafusion.DropViewNodeH\x00\x42\x11\n\x0fLogicalPlanType\"Q\n\x14LogicalExtensionNode\x12\x0c\n\x04node\x18\x01 \x01(\x0c\x12+\n\x06inputs\x18\x02 \x03(\x0b\x32\x1b.datafusion.LogicalPlanNode\"$\n\x11ProjectionColumns\x12\x0f\n\x07\x63olumns\x18\x01 \x03(\t\"2\n\tCsvFormat\x12\x12\n\nhas_header\x18\x01 \x01(\x08\x12\x11\n\tdelimiter\x18\x02 \x01(\t\"\x15\n\rParquetFormatJ\x04\x08\x01\x10\x02\"\x0c\n\nAvroFormat\"T\n\x19LogicalExprNodeCollection\x12\x37\n\x12logical_expr_nodes\x18\x01 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\"\x9a\x04\n\x14ListingTableScanNode\x12\x33\n\ntable_name\x18\x0e \x01(\x0b\x32\x1f.datafusion.OwnedTableReference\x12\r\n\x05paths\x18\x02 \x03(\t\x12\x16\n\x0e\x66ile_extension\x18\x03 \x01(\t\x12\x31\n\nprojection\x18\x04 \x01(\x0b\x32\x1d.datafusion.ProjectionColumns\x12\"\n\x06schema\x18\x05 \x01(\x0b\x32\x12.datafusion.Schema\x12,\n\x07\x66ilters\x18\x06 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\x12\x1c\n\x14table_partition_cols\x18\x07 \x03(\t\x12\x14\n\x0c\x63ollect_stat\x18\x08 \x01(\x08\x12\x19\n\x11target_partitions\x18\t \x01(\r\x12$\n\x03\x63sv\x18\n \x01(\x0b\x32\x15.datafusion.CsvFormatH\x00\x12,\n\x07parquet\x18\x0b \x01(\x0b\x32\x19.datafusion.ParquetFormatH\x00\x12&\n\x04\x61vro\x18\x0c \x01(\x0b\x32\x16.datafusion.AvroFormatH\x00\x12>\n\x0f\x66ile_sort_order\x18\r \x03(\x0b\x32%.datafusion.LogicalExprNodeCollectionB\x10\n\x0e\x46ileFormatTypeJ\x04\x08\x01\x10\x02\"\xe5\x01\n\x11ViewTableScanNode\x12\x33\n\ntable_name\x18\x06 \x01(\x0b\x32\x1f.datafusion.OwnedTableReference\x12*\n\x05input\x18\x02 \x01(\x0b\x32\x1b.datafusion.LogicalPlanNode\x12\"\n\x06schema\x18\x03 \x01(\x0b\x32\x12.datafusion.Schema\x12\x31\n\nprojection\x18\x04 \x01(\x0b\x32\x1d.datafusion.ProjectionColumns\x12\x12\n\ndefinition\x18\x05 \x01(\tJ\x04\x08\x01\x10\x02\"\xf0\x01\n\x13\x43ustomTableScanNode\x12\x33\n\ntable_name\x18\x06 \x01(\x0b\x32\x1f.datafusion.OwnedTableReference\x12\x31\n\nprojection\x18\x02 \x01(\x0b\x32\x1d.datafusion.ProjectionColumns\x12\"\n\x06schema\x18\x03 \x01(\x0b\x32\x12.datafusion.Schema\x12,\n\x07\x66ilters\x18\x04 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\x12\x19\n\x11\x63ustom_table_data\x18\x05 \x01(\x0cJ\x04\x08\x01\x10\x02\"\x8a\x01\n\x0eProjectionNode\x12*\n\x05input\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalPlanNode\x12)\n\x04\x65xpr\x18\x02 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\x12\x0f\n\x05\x61lias\x18\x03 \x01(\tH\x00\x42\x10\n\x0eoptional_alias\"f\n\rSelectionNode\x12*\n\x05input\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalPlanNode\x12)\n\x04\x65xpr\x18\x02 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\"p\n\x08SortNode\x12*\n\x05input\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalPlanNode\x12)\n\x04\x65xpr\x18\x02 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\x12\r\n\x05\x66\x65tch\x18\x03 \x01(\x03\"\x95\x01\n\x0fRepartitionNode\x12*\n\x05input\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalPlanNode\x12\x15\n\x0bround_robin\x18\x02 \x01(\x04H\x00\x12+\n\x04hash\x18\x03 \x01(\x0b\x32\x1b.datafusion.HashRepartitionH\x00\x42\x12\n\x10partition_method\"Z\n\x0fHashRepartition\x12.\n\thash_expr\x18\x01 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\x12\x17\n\x0fpartition_count\x18\x02 \x01(\x04\",\n\x11\x45mptyRelationNode\x12\x17\n\x0fproduce_one_row\x18\x01 \x01(\x08\"\xea\x03\n\x17\x43reateExternalTableNode\x12-\n\x04name\x18\x0c \x01(\x0b\x32\x1f.datafusion.OwnedTableReference\x12\x10\n\x08location\x18\x02 \x01(\t\x12\x11\n\tfile_type\x18\x03 \x01(\t\x12\x12\n\nhas_header\x18\x04 \x01(\x08\x12$\n\x06schema\x18\x05 \x01(\x0b\x32\x14.datafusion.DfSchema\x12\x1c\n\x14table_partition_cols\x18\x06 \x03(\t\x12\x15\n\rif_not_exists\x18\x07 \x01(\x08\x12\x11\n\tdelimiter\x18\x08 \x01(\t\x12\x12\n\ndefinition\x18\t \x01(\t\x12\x1d\n\x15\x66ile_compression_type\x18\n \x01(\t\x12:\n\x0border_exprs\x18\r \x03(\x0b\x32%.datafusion.LogicalExprNodeCollection\x12\x11\n\tunbounded\x18\x0e \x01(\x08\x12\x41\n\x07options\x18\x0b \x03(\x0b\x32\x30.datafusion.CreateExternalTableNode.OptionsEntry\x1a.\n\x0cOptionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01J\x04\x08\x01\x10\x02\"r\n\x0bPrepareNode\x12\x0c\n\x04name\x18\x01 \x01(\t\x12)\n\ndata_types\x18\x02 \x03(\x0b\x32\x15.datafusion.ArrowType\x12*\n\x05input\x18\x03 \x01(\x0b\x32\x1b.datafusion.LogicalPlanNode\"k\n\x17\x43reateCatalogSchemaNode\x12\x13\n\x0bschema_name\x18\x01 \x01(\t\x12\x15\n\rif_not_exists\x18\x02 \x01(\x08\x12$\n\x06schema\x18\x03 \x01(\x0b\x32\x14.datafusion.DfSchema\"f\n\x11\x43reateCatalogNode\x12\x14\n\x0c\x63\x61talog_name\x18\x01 \x01(\t\x12\x15\n\rif_not_exists\x18\x02 \x01(\x08\x12$\n\x06schema\x18\x03 \x01(\x0b\x32\x14.datafusion.DfSchema\"v\n\x0c\x44ropViewNode\x12-\n\x04name\x18\x01 \x01(\x0b\x32\x1f.datafusion.OwnedTableReference\x12\x11\n\tif_exists\x18\x02 \x01(\x08\x12$\n\x06schema\x18\x03 \x01(\x0b\x32\x14.datafusion.DfSchema\"\x99\x01\n\x0e\x43reateViewNode\x12-\n\x04name\x18\x05 \x01(\x0b\x32\x1f.datafusion.OwnedTableReference\x12*\n\x05input\x18\x02 \x01(\x0b\x32\x1b.datafusion.LogicalPlanNode\x12\x12\n\nor_replace\x18\x03 \x01(\x08\x12\x12\n\ndefinition\x18\x04 \x01(\tJ\x04\x08\x01\x10\x02\"N\n\nValuesNode\x12\x0e\n\x06n_cols\x18\x01 \x01(\x04\x12\x30\n\x0bvalues_list\x18\x02 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\"J\n\x0b\x41nalyzeNode\x12*\n\x05input\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalPlanNode\x12\x0f\n\x07verbose\x18\x02 \x01(\x08\"J\n\x0b\x45xplainNode\x12*\n\x05input\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalPlanNode\x12\x0f\n\x07verbose\x18\x02 \x01(\x08\"\x9c\x01\n\rAggregateNode\x12*\n\x05input\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalPlanNode\x12/\n\ngroup_expr\x18\x02 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\x12.\n\taggr_expr\x18\x03 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\"j\n\nWindowNode\x12*\n\x05input\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalPlanNode\x12\x30\n\x0bwindow_expr\x18\x02 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\"\xef\x02\n\x08JoinNode\x12)\n\x04left\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalPlanNode\x12*\n\x05right\x18\x02 \x01(\x0b\x32\x1b.datafusion.LogicalPlanNode\x12\'\n\tjoin_type\x18\x03 \x01(\x0e\x32\x14.datafusion.JoinType\x12\x33\n\x0fjoin_constraint\x18\x04 \x01(\x0e\x32\x1a.datafusion.JoinConstraint\x12\x32\n\rleft_join_key\x18\x05 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\x12\x33\n\x0eright_join_key\x18\x06 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\x12\x18\n\x10null_equals_null\x18\x07 \x01(\x08\x12+\n\x06\x66ilter\x18\x08 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\":\n\x0c\x44istinctNode\x12*\n\x05input\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalPlanNode\"8\n\tUnionNode\x12+\n\x06inputs\x18\x01 \x03(\x0b\x32\x1b.datafusion.LogicalPlanNode\"f\n\rCrossJoinNode\x12)\n\x04left\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalPlanNode\x12*\n\x05right\x18\x02 \x01(\x0b\x32\x1b.datafusion.LogicalPlanNode\"T\n\tLimitNode\x12*\n\x05input\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalPlanNode\x12\x0c\n\x04skip\x18\x02 \x01(\x03\x12\r\n\x05\x66\x65tch\x18\x03 \x01(\x03\">\n\x11SelectionExecNode\x12)\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\"u\n\x11SubqueryAliasNode\x12*\n\x05input\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalPlanNode\x12.\n\x05\x61lias\x18\x03 \x01(\x0b\x32\x1f.datafusion.OwnedTableReferenceJ\x04\x08\x02\x10\x03\"\xad\x0c\n\x0fLogicalExprNode\x12$\n\x06\x63olumn\x18\x01 \x01(\x0b\x32\x12.datafusion.ColumnH\x00\x12&\n\x05\x61lias\x18\x02 \x01(\x0b\x32\x15.datafusion.AliasNodeH\x00\x12*\n\x07literal\x18\x03 \x01(\x0b\x32\x17.datafusion.ScalarValueH\x00\x12\x31\n\x0b\x62inary_expr\x18\x04 \x01(\x0b\x32\x1a.datafusion.BinaryExprNodeH\x00\x12\x37\n\x0e\x61ggregate_expr\x18\x05 \x01(\x0b\x32\x1d.datafusion.AggregateExprNodeH\x00\x12*\n\x0cis_null_expr\x18\x06 \x01(\x0b\x32\x12.datafusion.IsNullH\x00\x12\x31\n\x10is_not_null_expr\x18\x07 \x01(\x0b\x32\x15.datafusion.IsNotNullH\x00\x12#\n\x08not_expr\x18\x08 \x01(\x0b\x32\x0f.datafusion.NotH\x00\x12*\n\x07\x62\x65tween\x18\t \x01(\x0b\x32\x17.datafusion.BetweenNodeH\x00\x12%\n\x05\x63\x61se_\x18\n \x01(\x0b\x32\x14.datafusion.CaseNodeH\x00\x12$\n\x04\x63\x61st\x18\x0b \x01(\x0b\x32\x14.datafusion.CastNodeH\x00\x12(\n\x04sort\x18\x0c \x01(\x0b\x32\x18.datafusion.SortExprNodeH\x00\x12,\n\x08negative\x18\r \x01(\x0b\x32\x18.datafusion.NegativeNodeH\x00\x12)\n\x07in_list\x18\x0e \x01(\x0b\x32\x16.datafusion.InListNodeH\x00\x12\x12\n\x08wildcard\x18\x0f \x01(\x08H\x00\x12\x39\n\x0fscalar_function\x18\x10 \x01(\x0b\x32\x1e.datafusion.ScalarFunctionNodeH\x00\x12+\n\x08try_cast\x18\x11 \x01(\x0b\x32\x17.datafusion.TryCastNodeH\x00\x12\x31\n\x0bwindow_expr\x18\x12 \x01(\x0b\x32\x1a.datafusion.WindowExprNodeH\x00\x12>\n\x12\x61ggregate_udf_expr\x18\x13 \x01(\x0b\x32 .datafusion.AggregateUDFExprNodeH\x00\x12\x38\n\x0fscalar_udf_expr\x18\x14 \x01(\x0b\x32\x1d.datafusion.ScalarUDFExprNodeH\x00\x12\x38\n\x11get_indexed_field\x18\x15 \x01(\x0b\x32\x1b.datafusion.GetIndexedFieldH\x00\x12\x33\n\x0cgrouping_set\x18\x16 \x01(\x0b\x32\x1b.datafusion.GroupingSetNodeH\x00\x12$\n\x04\x63ube\x18\x17 \x01(\x0b\x32\x14.datafusion.CubeNodeH\x00\x12(\n\x06rollup\x18\x18 \x01(\x0b\x32\x16.datafusion.RollupNodeH\x00\x12%\n\x07is_true\x18\x19 \x01(\x0b\x32\x12.datafusion.IsTrueH\x00\x12\'\n\x08is_false\x18\x1a \x01(\x0b\x32\x13.datafusion.IsFalseH\x00\x12+\n\nis_unknown\x18\x1b \x01(\x0b\x32\x15.datafusion.IsUnknownH\x00\x12,\n\x0bis_not_true\x18\x1c \x01(\x0b\x32\x15.datafusion.IsNotTrueH\x00\x12.\n\x0cis_not_false\x18\x1d \x01(\x0b\x32\x16.datafusion.IsNotFalseH\x00\x12\x32\n\x0eis_not_unknown\x18\x1e \x01(\x0b\x32\x18.datafusion.IsNotUnknownH\x00\x12$\n\x04like\x18\x1f \x01(\x0b\x32\x14.datafusion.LikeNodeH\x00\x12&\n\x05ilike\x18  \x01(\x0b\x32\x15.datafusion.ILikeNodeH\x00\x12/\n\nsimilar_to\x18! \x01(\x0b\x32\x19.datafusion.SimilarToNodeH\x00\x12\x32\n\x0bplaceholder\x18\" \x01(\x0b\x32\x1b.datafusion.PlaceholderNodeH\x00\x42\n\n\x08\x45xprType\"G\n\x0fPlaceholderNode\x12\n\n\x02id\x18\x01 \x01(\t\x12(\n\tdata_type\x18\x02 \x01(\x0b\x32\x15.datafusion.ArrowType\"<\n\x0fLogicalExprList\x12)\n\x04\x65xpr\x18\x01 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\"<\n\x0fGroupingSetNode\x12)\n\x04\x65xpr\x18\x01 \x03(\x0b\x32\x1b.datafusion.LogicalExprList\"5\n\x08\x43ubeNode\x12)\n\x04\x65xpr\x18\x01 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\"7\n\nRollupNode\x12)\n\x04\x65xpr\x18\x01 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\"b\n\x0fGetIndexedField\x12)\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\x12$\n\x03key\x18\x02 \x01(\x0b\x32\x17.datafusion.ScalarValue\"3\n\x06IsNull\x12)\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\"6\n\tIsNotNull\x12)\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\"3\n\x06IsTrue\x12)\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\"4\n\x07IsFalse\x12)\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\"6\n\tIsUnknown\x12)\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\"6\n\tIsNotTrue\x12)\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\"7\n\nIsNotFalse\x12)\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\"9\n\x0cIsNotUnknown\x12)\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\"0\n\x03Not\x12)\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\"E\n\tAliasNode\x12)\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\x12\r\n\x05\x61lias\x18\x02 \x01(\t\"K\n\x0e\x42inaryExprNode\x12-\n\x08operands\x18\x01 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\x12\n\n\x02op\x18\x03 \x01(\t\"9\n\x0cNegativeNode\x12)\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\"s\n\nInListNode\x12)\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\x12)\n\x04list\x18\x02 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\x12\x0f\n\x07negated\x18\x03 \x01(\x08\"h\n\x12ScalarFunctionNode\x12\'\n\x03\x66un\x18\x01 \x01(\x0e\x32\x1a.datafusion.ScalarFunction\x12)\n\x04\x61rgs\x18\x02 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\"\xe2\x01\n\x11\x41ggregateExprNode\x12\x34\n\raggr_function\x18\x01 \x01(\x0e\x32\x1d.datafusion.AggregateFunction\x12)\n\x04\x65xpr\x18\x02 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\x12\x10\n\x08\x64istinct\x18\x03 \x01(\x08\x12+\n\x06\x66ilter\x18\x04 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\x12-\n\x08order_by\x18\x05 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\"\xaf\x01\n\x14\x41ggregateUDFExprNode\x12\x10\n\x08\x66un_name\x18\x01 \x01(\t\x12)\n\x04\x61rgs\x18\x02 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\x12+\n\x06\x66ilter\x18\x03 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\x12-\n\x08order_by\x18\x04 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\"P\n\x11ScalarUDFExprNode\x12\x10\n\x08\x66un_name\x18\x01 \x01(\t\x12)\n\x04\x61rgs\x18\x02 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\"\xf7\x02\n\x0eWindowExprNode\x12\x36\n\raggr_function\x18\x01 \x01(\x0e\x32\x1d.datafusion.AggregateFunctionH\x00\x12>\n\x11\x62uilt_in_function\x18\x02 \x01(\x0e\x32!.datafusion.BuiltInWindowFunctionH\x00\x12\x0e\n\x04udaf\x18\x03 \x01(\tH\x00\x12\x0e\n\x04udwf\x18\t \x01(\tH\x00\x12)\n\x04\x65xpr\x18\x04 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\x12\x31\n\x0cpartition_by\x18\x05 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\x12-\n\x08order_by\x18\x06 \x03(\x0b\x32\x1b.datafusion.LogicalExprNode\x12-\n\x0cwindow_frame\x18\x08 \x01(\x0b\x32\x17.datafusion.WindowFrameB\x11\n\x0fwindow_function\"\x9e\x01\n\x0b\x42\x65tweenNode\x12)\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\x12\x0f\n\x07negated\x18\x02 \x01(\x08\x12(\n\x03low\x18\x03 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\x12)\n\x04high\x18\x04 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\"\x89\x01\n\x08LikeNode\x12\x0f\n\x07negated\x18\x01 \x01(\x08\x12)\n\x04\x65xpr\x18\x02 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\x12,\n\x07pattern\x18\x03 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\x12\x13\n\x0b\x65scape_char\x18\x04 \x01(\t\"\x8a\x01\n\tILikeNode\x12\x0f\n\x07negated\x18\x01 \x01(\x08\x12)\n\x04\x65xpr\x18\x02 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\x12,\n\x07pattern\x18\x03 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\x12\x13\n\x0b\x65scape_char\x18\x04 \x01(\t\"\x8e\x01\n\rSimilarToNode\x12\x0f\n\x07negated\x18\x01 \x01(\x08\x12)\n\x04\x65xpr\x18\x02 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\x12,\n\x07pattern\x18\x03 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\x12\x13\n\x0b\x65scape_char\x18\x04 \x01(\t\"\x93\x01\n\x08\x43\x61seNode\x12)\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\x12,\n\x0ewhen_then_expr\x18\x02 \x03(\x0b\x32\x14.datafusion.WhenThen\x12.\n\telse_expr\x18\x03 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\"j\n\x08WhenThen\x12.\n\twhen_expr\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\x12.\n\tthen_expr\x18\x02 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\"`\n\x08\x43\x61stNode\x12)\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\x12)\n\narrow_type\x18\x02 \x01(\x0b\x32\x15.datafusion.ArrowType\"c\n\x0bTryCastNode\x12)\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\x12)\n\narrow_type\x18\x02 \x01(\x0b\x32\x15.datafusion.ArrowType\"[\n\x0cSortExprNode\x12)\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1b.datafusion.LogicalExprNode\x12\x0b\n\x03\x61sc\x18\x02 \x01(\x08\x12\x13\n\x0bnulls_first\x18\x03 \x01(\x08\"\xb6\x01\n\x0bWindowFrame\x12\x38\n\x12window_frame_units\x18\x01 \x01(\x0e\x32\x1c.datafusion.WindowFrameUnits\x12\x31\n\x0bstart_bound\x18\x02 \x01(\x0b\x32\x1c.datafusion.WindowFrameBound\x12-\n\x05\x62ound\x18\x03 \x01(\x0b\x32\x1c.datafusion.WindowFrameBoundH\x00\x42\x0b\n\tend_bound\"\x83\x01\n\x10WindowFrameBound\x12\x41\n\x17window_frame_bound_type\x18\x01 \x01(\x0e\x32 .datafusion.WindowFrameBoundType\x12,\n\x0b\x62ound_value\x18\x02 \x01(\x0b\x32\x17.datafusion.ScalarValue\"\x91\x01\n\x06Schema\x12\"\n\x07\x63olumns\x18\x01 \x03(\x0b\x32\x11.datafusion.Field\x12\x32\n\x08metadata\x18\x02 \x03(\x0b\x32 .datafusion.Schema.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xdb\x01\n\x05\x46ield\x12\x0c\n\x04name\x18\x01 \x01(\t\x12)\n\narrow_type\x18\x02 \x01(\x0b\x32\x15.datafusion.ArrowType\x12\x10\n\x08nullable\x18\x03 \x01(\x08\x12#\n\x08\x63hildren\x18\x04 \x03(\x0b\x32\x11.datafusion.Field\x12\x31\n\x08metadata\x18\x05 \x03(\x0b\x32\x1f.datafusion.Field.MetadataEntry\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"!\n\x0f\x46ixedSizeBinary\x12\x0e\n\x06length\x18\x01 \x01(\x05\"F\n\tTimestamp\x12\'\n\ttime_unit\x18\x01 \x01(\x0e\x32\x14.datafusion.TimeUnit\x12\x10\n\x08timezone\x18\x02 \x01(\t\"7\n\x07\x44\x65\x63imal\x12\x11\n\tprecision\x18\x03 \x01(\r\x12\r\n\x05scale\x18\x04 \x01(\x05J\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03\"-\n\x04List\x12%\n\nfield_type\x18\x01 \x01(\x0b\x32\x11.datafusion.Field\"I\n\rFixedSizeList\x12%\n\nfield_type\x18\x01 \x01(\x0b\x32\x11.datafusion.Field\x12\x11\n\tlist_size\x18\x02 \x01(\x05\"V\n\nDictionary\x12\"\n\x03key\x18\x01 \x01(\x0b\x32\x15.datafusion.ArrowType\x12$\n\x05value\x18\x02 \x01(\x0b\x32\x15.datafusion.ArrowType\"4\n\x06Struct\x12*\n\x0fsub_field_types\x18\x01 \x03(\x0b\x32\x11.datafusion.Field\"A\n\x03Map\x12%\n\nfield_type\x18\x01 \x01(\x0b\x32\x11.datafusion.Field\x12\x13\n\x0bkeys_sorted\x18\x02 \x01(\x08\"l\n\x05Union\x12&\n\x0bunion_types\x18\x01 \x03(\x0b\x32\x11.datafusion.Field\x12)\n\nunion_mode\x18\x02 \x01(\x0e\x32\x15.datafusion.UnionMode\x12\x10\n\x08type_ids\x18\x03 \x03(\x05\"m\n\x0fScalarListValue\x12\x0f\n\x07is_null\x18\x03 \x01(\x08\x12 \n\x05\x66ield\x18\x01 \x01(\x0b\x32\x11.datafusion.Field\x12\'\n\x06values\x18\x02 \x03(\x0b\x32\x17.datafusion.ScalarValue\"_\n\x11ScalarTime32Value\x12\x1d\n\x13time32_second_value\x18\x01 \x01(\x05H\x00\x12\"\n\x18time32_millisecond_value\x18\x02 \x01(\x05H\x00\x42\x07\n\x05value\"c\n\x11ScalarTime64Value\x12\"\n\x18time64_microsecond_value\x18\x01 \x01(\x03H\x00\x12!\n\x17time64_nanosecond_value\x18\x02 \x01(\x03H\x00\x42\x07\n\x05value\"\xb3\x01\n\x14ScalarTimestampValue\x12 \n\x16time_microsecond_value\x18\x01 \x01(\x03H\x00\x12\x1f\n\x15time_nanosecond_value\x18\x02 \x01(\x03H\x00\x12\x1b\n\x11time_second_value\x18\x03 \x01(\x03H\x00\x12 \n\x16time_millisecond_value\x18\x04 \x01(\x03H\x00\x12\x10\n\x08timezone\x18\x05 \x01(\tB\x07\n\x05value\"j\n\x15ScalarDictionaryValue\x12)\n\nindex_type\x18\x01 \x01(\x0b\x32\x15.datafusion.ArrowType\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.datafusion.ScalarValue\"H\n\x19IntervalMonthDayNanoValue\x12\x0e\n\x06months\x18\x01 \x01(\x05\x12\x0c\n\x04\x64\x61ys\x18\x02 \x01(\x05\x12\r\n\x05nanos\x18\x03 \x01(\x03\"_\n\x0bStructValue\x12-\n\x0c\x66ield_values\x18\x02 \x03(\x0b\x32\x17.datafusion.ScalarValue\x12!\n\x06\x66ields\x18\x03 \x03(\x0b\x32\x11.datafusion.Field\"7\n\x15ScalarFixedSizeBinary\x12\x0e\n\x06values\x18\x01 \x01(\x0c\x12\x0e\n\x06length\x18\x02 \x01(\x05\"\xcf\t\n\x0bScalarValue\x12+\n\nnull_value\x18! \x01(\x0b\x32\x15.datafusion.ArrowTypeH\x00\x12\x14\n\nbool_value\x18\x01 \x01(\x08H\x00\x12\x14\n\nutf8_value\x18\x02 \x01(\tH\x00\x12\x1a\n\x10large_utf8_value\x18\x03 \x01(\tH\x00\x12\x14\n\nint8_value\x18\x04 \x01(\x05H\x00\x12\x15\n\x0bint16_value\x18\x05 \x01(\x05H\x00\x12\x15\n\x0bint32_value\x18\x06 \x01(\x05H\x00\x12\x15\n\x0bint64_value\x18\x07 \x01(\x03H\x00\x12\x15\n\x0buint8_value\x18\x08 \x01(\rH\x00\x12\x16\n\x0cuint16_value\x18\t \x01(\rH\x00\x12\x16\n\x0cuint32_value\x18\n \x01(\rH\x00\x12\x16\n\x0cuint64_value\x18\x0b \x01(\x04H\x00\x12\x17\n\rfloat32_value\x18\x0c \x01(\x02H\x00\x12\x17\n\rfloat64_value\x18\r \x01(\x01H\x00\x12\x17\n\rdate_32_value\x18\x0e \x01(\x05H\x00\x12\x35\n\x0ctime32_value\x18\x0f \x01(\x0b\x32\x1d.datafusion.ScalarTime32ValueH\x00\x12\x31\n\nlist_value\x18\x11 \x01(\x0b\x32\x1b.datafusion.ScalarListValueH\x00\x12\x32\n\x10\x64\x65\x63imal128_value\x18\x14 \x01(\x0b\x32\x16.datafusion.Decimal128H\x00\x12\x17\n\rdate_64_value\x18\x15 \x01(\x03H\x00\x12\"\n\x18interval_yearmonth_value\x18\x18 \x01(\x05H\x00\x12 \n\x16interval_daytime_value\x18\x19 \x01(\x03H\x00\x12\x1f\n\x15\x64uration_second_value\x18# \x01(\x03H\x00\x12$\n\x1a\x64uration_millisecond_value\x18$ \x01(\x03H\x00\x12$\n\x1a\x64uration_microsecond_value\x18% \x01(\x03H\x00\x12#\n\x19\x64uration_nanosecond_value\x18& \x01(\x03H\x00\x12;\n\x0ftimestamp_value\x18\x1a \x01(\x0b\x32 .datafusion.ScalarTimestampValueH\x00\x12=\n\x10\x64ictionary_value\x18\x1b \x01(\x0b\x32!.datafusion.ScalarDictionaryValueH\x00\x12\x16\n\x0c\x62inary_value\x18\x1c \x01(\x0cH\x00\x12\x1c\n\x12large_binary_value\x18\x1d \x01(\x0cH\x00\x12\x35\n\x0ctime64_value\x18\x1e \x01(\x0b\x32\x1d.datafusion.ScalarTime64ValueH\x00\x12H\n\x17interval_month_day_nano\x18\x1f \x01(\x0b\x32%.datafusion.IntervalMonthDayNanoValueH\x00\x12/\n\x0cstruct_value\x18  \x01(\x0b\x32\x17.datafusion.StructValueH\x00\x12\x44\n\x17\x66ixed_size_binary_value\x18\" \x01(\x0b\x32!.datafusion.ScalarFixedSizeBinaryH\x00\x42\x07\n\x05valueJ\x04\x08\x13\x10\x14\"1\n\nDecimal128\x12\r\n\x05value\x18\x01 \x01(\x0c\x12\t\n\x01p\x18\x02 \x01(\x03\x12\t\n\x01s\x18\x03 \x01(\x03\"\x94\x0b\n\tArrowType\x12(\n\x04NONE\x18\x01 \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12(\n\x04\x42OOL\x18\x02 \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12)\n\x05UINT8\x18\x03 \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12(\n\x04INT8\x18\x04 \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12*\n\x06UINT16\x18\x05 \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12)\n\x05INT16\x18\x06 \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12*\n\x06UINT32\x18\x07 \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12)\n\x05INT32\x18\x08 \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12*\n\x06UINT64\x18\t \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12)\n\x05INT64\x18\n \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12+\n\x07\x46LOAT16\x18\x0b \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12+\n\x07\x46LOAT32\x18\x0c \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12+\n\x07\x46LOAT64\x18\r \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12(\n\x04UTF8\x18\x0e \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12.\n\nLARGE_UTF8\x18  \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12*\n\x06\x42INARY\x18\x0f \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12\x1b\n\x11\x46IXED_SIZE_BINARY\x18\x10 \x01(\x05H\x00\x12\x30\n\x0cLARGE_BINARY\x18\x1f \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12*\n\x06\x44\x41TE32\x18\x11 \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12*\n\x06\x44\x41TE64\x18\x12 \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12(\n\x08\x44URATION\x18\x13 \x01(\x0e\x32\x14.datafusion.TimeUnitH\x00\x12*\n\tTIMESTAMP\x18\x14 \x01(\x0b\x32\x15.datafusion.TimestampH\x00\x12&\n\x06TIME32\x18\x15 \x01(\x0e\x32\x14.datafusion.TimeUnitH\x00\x12&\n\x06TIME64\x18\x16 \x01(\x0e\x32\x14.datafusion.TimeUnitH\x00\x12,\n\x08INTERVAL\x18\x17 \x01(\x0e\x32\x18.datafusion.IntervalUnitH\x00\x12&\n\x07\x44\x45\x43IMAL\x18\x18 \x01(\x0b\x32\x13.datafusion.DecimalH\x00\x12 \n\x04LIST\x18\x19 \x01(\x0b\x32\x10.datafusion.ListH\x00\x12&\n\nLARGE_LIST\x18\x1a \x01(\x0b\x32\x10.datafusion.ListH\x00\x12\x34\n\x0f\x46IXED_SIZE_LIST\x18\x1b \x01(\x0b\x32\x19.datafusion.FixedSizeListH\x00\x12$\n\x06STRUCT\x18\x1c \x01(\x0b\x32\x12.datafusion.StructH\x00\x12\"\n\x05UNION\x18\x1d \x01(\x0b\x32\x11.datafusion.UnionH\x00\x12,\n\nDICTIONARY\x18\x1e \x01(\x0b\x32\x16.datafusion.DictionaryH\x00\x12\x1e\n\x03MAP\x18! \x01(\x0b\x32\x0f.datafusion.MapH\x00\x42\x11\n\x0f\x61rrow_type_enum\"\x0e\n\x0c\x45mptyMessage\"0\n\x17\x41nalyzedLogicalPlanType\x12\x15\n\ranalyzer_name\x18\x01 \x01(\t\"2\n\x18OptimizedLogicalPlanType\x12\x16\n\x0eoptimizer_name\x18\x01 \x01(\t\"3\n\x19OptimizedPhysicalPlanType\x12\x16\n\x0eoptimizer_name\x18\x01 \x01(\t\"\x8a\x04\n\x08PlanType\x12\x36\n\x12InitialLogicalPlan\x18\x01 \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12\x42\n\x13\x41nalyzedLogicalPlan\x18\x07 \x01(\x0b\x32#.datafusion.AnalyzedLogicalPlanTypeH\x00\x12<\n\x18\x46inalAnalyzedLogicalPlan\x18\x08 \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12\x44\n\x14OptimizedLogicalPlan\x18\x02 \x01(\x0b\x32$.datafusion.OptimizedLogicalPlanTypeH\x00\x12\x34\n\x10\x46inalLogicalPlan\x18\x03 \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12\x37\n\x13InitialPhysicalPlan\x18\x04 \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x12\x46\n\x15OptimizedPhysicalPlan\x18\x05 \x01(\x0b\x32%.datafusion.OptimizedPhysicalPlanTypeH\x00\x12\x35\n\x11\x46inalPhysicalPlan\x18\x06 \x01(\x0b\x32\x18.datafusion.EmptyMessageH\x00\x42\x10\n\x0eplan_type_enum\"H\n\x0fStringifiedPlan\x12\'\n\tplan_type\x18\x01 \x01(\x0b\x32\x14.datafusion.PlanType\x12\x0c\n\x04plan\x18\x02 \x01(\t\"#\n\x12\x42\x61reTableReference\x12\r\n\x05table\x18\x01 \x01(\t\"6\n\x15PartialTableReference\x12\x0e\n\x06schema\x18\x01 \x01(\t\x12\r\n\x05table\x18\x02 \x01(\t\"D\n\x12\x46ullTableReference\x12\x0f\n\x07\x63\x61talog\x18\x01 \x01(\t\x12\x0e\n\x06schema\x18\x02 \x01(\t\x12\r\n\x05table\x18\x03 \x01(\t\"\xc3\x01\n\x13OwnedTableReference\x12.\n\x04\x62\x61re\x18\x01 \x01(\x0b\x32\x1e.datafusion.BareTableReferenceH\x00\x12\x34\n\x07partial\x18\x02 \x01(\x0b\x32!.datafusion.PartialTableReferenceH\x00\x12.\n\x04\x66ull\x18\x03 \x01(\x0b\x32\x1e.datafusion.FullTableReferenceH\x00\x42\x16\n\x14table_reference_enum\"\x8a\t\n\x10PhysicalPlanNode\x12\x37\n\x0cparquet_scan\x18\x01 \x01(\x0b\x32\x1f.datafusion.ParquetScanExecNodeH\x00\x12/\n\x08\x63sv_scan\x18\x02 \x01(\x0b\x32\x1b.datafusion.CsvScanExecNodeH\x00\x12*\n\x05\x65mpty\x18\x03 \x01(\x0b\x32\x19.datafusion.EmptyExecNodeH\x00\x12\x34\n\nprojection\x18\x04 \x01(\x0b\x32\x1e.datafusion.ProjectionExecNodeH\x00\x12\x37\n\x0cglobal_limit\x18\x06 \x01(\x0b\x32\x1f.datafusion.GlobalLimitExecNodeH\x00\x12\x35\n\x0blocal_limit\x18\x07 \x01(\x0b\x32\x1e.datafusion.LocalLimitExecNodeH\x00\x12\x32\n\taggregate\x18\x08 \x01(\x0b\x32\x1d.datafusion.AggregateExecNodeH\x00\x12\x31\n\thash_join\x18\t \x01(\x0b\x32\x1c.datafusion.HashJoinExecNodeH\x00\x12(\n\x04sort\x18\n \x01(\x0b\x32\x18.datafusion.SortExecNodeH\x00\x12?\n\x10\x63oalesce_batches\x18\x0b \x01(\x0b\x32#.datafusion.CoalesceBatchesExecNodeH\x00\x12,\n\x06\x66ilter\x18\x0c \x01(\x0b\x32\x1a.datafusion.FilterExecNodeH\x00\x12\x37\n\x05merge\x18\r \x01(\x0b\x32&.datafusion.CoalescePartitionsExecNodeH\x00\x12\x36\n\x0brepartition\x18\x0e \x01(\x0b\x32\x1f.datafusion.RepartitionExecNodeH\x00\x12/\n\x06window\x18\x0f \x01(\x0b\x32\x1d.datafusion.WindowAggExecNodeH\x00\x12\x33\n\ncross_join\x18\x10 \x01(\x0b\x32\x1d.datafusion.CrossJoinExecNodeH\x00\x12\x31\n\tavro_scan\x18\x11 \x01(\x0b\x32\x1c.datafusion.AvroScanExecNodeH\x00\x12\x36\n\textension\x18\x12 \x01(\x0b\x32!.datafusion.PhysicalExtensionNodeH\x00\x12*\n\x05union\x18\x13 \x01(\x0b\x32\x19.datafusion.UnionExecNodeH\x00\x12.\n\x07\x65xplain\x18\x14 \x01(\x0b\x32\x1b.datafusion.ExplainExecNodeH\x00\x12H\n\x15sort_preserving_merge\x18\x15 \x01(\x0b\x32\'.datafusion.SortPreservingMergeExecNodeH\x00\x12>\n\x10nested_loop_join\x18\x16 \x01(\x0b\x32\".datafusion.NestedLoopJoinExecNodeH\x00\x42\x12\n\x10PhysicalPlanType\"S\n\x15PhysicalExtensionNode\x12\x0c\n\x04node\x18\x01 \x01(\x0c\x12,\n\x06inputs\x18\x02 \x03(\x0b\x32\x1c.datafusion.PhysicalPlanNode\"\xcb\x08\n\x10PhysicalExprNode\x12,\n\x06\x63olumn\x18\x01 \x01(\x0b\x32\x1a.datafusion.PhysicalColumnH\x00\x12*\n\x07literal\x18\x02 \x01(\x0b\x32\x17.datafusion.ScalarValueH\x00\x12\x39\n\x0b\x62inary_expr\x18\x03 \x01(\x0b\x32\".datafusion.PhysicalBinaryExprNodeH\x00\x12?\n\x0e\x61ggregate_expr\x18\x04 \x01(\x0b\x32%.datafusion.PhysicalAggregateExprNodeH\x00\x12\x32\n\x0cis_null_expr\x18\x05 \x01(\x0b\x32\x1a.datafusion.PhysicalIsNullH\x00\x12\x39\n\x10is_not_null_expr\x18\x06 \x01(\x0b\x32\x1d.datafusion.PhysicalIsNotNullH\x00\x12+\n\x08not_expr\x18\x07 \x01(\x0b\x32\x17.datafusion.PhysicalNotH\x00\x12-\n\x05\x63\x61se_\x18\x08 \x01(\x0b\x32\x1c.datafusion.PhysicalCaseNodeH\x00\x12,\n\x04\x63\x61st\x18\t \x01(\x0b\x32\x1c.datafusion.PhysicalCastNodeH\x00\x12\x30\n\x04sort\x18\n \x01(\x0b\x32 .datafusion.PhysicalSortExprNodeH\x00\x12\x34\n\x08negative\x18\x0b \x01(\x0b\x32 .datafusion.PhysicalNegativeNodeH\x00\x12\x31\n\x07in_list\x18\x0c \x01(\x0b\x32\x1e.datafusion.PhysicalInListNodeH\x00\x12\x41\n\x0fscalar_function\x18\r \x01(\x0b\x32&.datafusion.PhysicalScalarFunctionNodeH\x00\x12\x33\n\x08try_cast\x18\x0e \x01(\x0b\x32\x1f.datafusion.PhysicalTryCastNodeH\x00\x12\x39\n\x0bwindow_expr\x18\x0f \x01(\x0b\x32\".datafusion.PhysicalWindowExprNodeH\x00\x12\x37\n\nscalar_udf\x18\x10 \x01(\x0b\x32!.datafusion.PhysicalScalarUdfNodeH\x00\x12O\n\x17\x64\x61te_time_interval_expr\x18\x11 \x01(\x0b\x32,.datafusion.PhysicalDateTimeIntervalExprNodeH\x00\x12\x35\n\tlike_expr\x18\x12 \x01(\x0b\x32 .datafusion.PhysicalLikeExprNodeH\x00\x12M\n\x16get_indexed_field_expr\x18\x13 \x01(\x0b\x32+.datafusion.PhysicalGetIndexedFieldExprNodeH\x00\x42\n\n\x08\x45xprType\"}\n\x15PhysicalScalarUdfNode\x12\x0c\n\x04name\x18\x01 \x01(\t\x12*\n\x04\x61rgs\x18\x02 \x03(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12*\n\x0breturn_type\x18\x04 \x01(\x0b\x32\x15.datafusion.ArrowType\"\x84\x02\n\x19PhysicalAggregateExprNode\x12\x36\n\raggr_function\x18\x01 \x01(\x0e\x32\x1d.datafusion.AggregateFunctionH\x00\x12$\n\x1auser_defined_aggr_function\x18\x04 \x01(\tH\x00\x12*\n\x04\x65xpr\x18\x02 \x03(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12\x36\n\x0cordering_req\x18\x05 \x03(\x0b\x32 .datafusion.PhysicalSortExprNode\x12\x10\n\x08\x64istinct\x18\x03 \x01(\x08\x42\x13\n\x11\x41ggregateFunction\"\xcf\x01\n\x16PhysicalWindowExprNode\x12\x36\n\raggr_function\x18\x01 \x01(\x0e\x32\x1d.datafusion.AggregateFunctionH\x00\x12>\n\x11\x62uilt_in_function\x18\x02 \x01(\x0e\x32!.datafusion.BuiltInWindowFunctionH\x00\x12*\n\x04\x65xpr\x18\x04 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNodeB\x11\n\x0fwindow_function\"<\n\x0ePhysicalIsNull\x12*\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\"?\n\x11PhysicalIsNotNull\x12*\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\"9\n\x0bPhysicalNot\x12*\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\"N\n\x11PhysicalAliasNode\x12*\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12\r\n\x05\x61lias\x18\x02 \x01(\t\"v\n\x16PhysicalBinaryExprNode\x12\'\n\x01l\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12\'\n\x01r\x18\x02 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12\n\n\x02op\x18\x03 \x01(\t\"\x80\x01\n PhysicalDateTimeIntervalExprNode\x12\'\n\x01l\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12\'\n\x01r\x18\x02 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12\n\n\x02op\x18\x03 \x01(\t\"\x9c\x01\n\x14PhysicalLikeExprNode\x12\x0f\n\x07negated\x18\x01 \x01(\x08\x12\x18\n\x10\x63\x61se_insensitive\x18\x02 \x01(\x08\x12*\n\x04\x65xpr\x18\x03 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12-\n\x07pattern\x18\x04 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\"d\n\x14PhysicalSortExprNode\x12*\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12\x0b\n\x03\x61sc\x18\x02 \x01(\x08\x12\x13\n\x0bnulls_first\x18\x03 \x01(\x08\"t\n\x10PhysicalWhenThen\x12/\n\twhen_expr\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12/\n\tthen_expr\x18\x02 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\"}\n\x12PhysicalInListNode\x12*\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12*\n\x04list\x18\x02 \x03(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12\x0f\n\x07negated\x18\x03 \x01(\x08\"\xa5\x01\n\x10PhysicalCaseNode\x12*\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12\x34\n\x0ewhen_then_expr\x18\x02 \x03(\x0b\x32\x1c.datafusion.PhysicalWhenThen\x12/\n\telse_expr\x18\x03 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\"\xab\x01\n\x1aPhysicalScalarFunctionNode\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\'\n\x03\x66un\x18\x02 \x01(\x0e\x32\x1a.datafusion.ScalarFunction\x12*\n\x04\x61rgs\x18\x03 \x03(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12*\n\x0breturn_type\x18\x04 \x01(\x0b\x32\x15.datafusion.ArrowType\"l\n\x13PhysicalTryCastNode\x12*\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12)\n\narrow_type\x18\x02 \x01(\x0b\x32\x15.datafusion.ArrowType\"i\n\x10PhysicalCastNode\x12*\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12)\n\narrow_type\x18\x02 \x01(\x0b\x32\x15.datafusion.ArrowType\"B\n\x14PhysicalNegativeNode\x12*\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\"i\n\x0e\x46ilterExecNode\x12+\n\x05input\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalPlanNode\x12*\n\x04\x65xpr\x18\x02 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\"7\n\tFileGroup\x12*\n\x05\x66iles\x18\x01 \x03(\x0b\x32\x1b.datafusion.PartitionedFile\"\x1a\n\tScanLimit\x12\r\n\x05limit\x18\x01 \x01(\r\"d\n\x1ePhysicalSortExprNodeCollection\x12\x42\n\x18physical_sort_expr_nodes\x18\x01 \x03(\x0b\x32 .datafusion.PhysicalSortExprNode\"\xcb\x02\n\x10\x46ileScanExecConf\x12*\n\x0b\x66ile_groups\x18\x01 \x03(\x0b\x32\x15.datafusion.FileGroup\x12\"\n\x06schema\x18\x02 \x01(\x0b\x32\x12.datafusion.Schema\x12\x12\n\nprojection\x18\x04 \x03(\r\x12$\n\x05limit\x18\x05 \x01(\x0b\x32\x15.datafusion.ScanLimit\x12*\n\nstatistics\x18\x06 \x01(\x0b\x32\x16.datafusion.Statistics\x12\x1c\n\x14table_partition_cols\x18\x07 \x03(\t\x12\x18\n\x10object_store_url\x18\x08 \x01(\t\x12\x43\n\x0foutput_ordering\x18\t \x03(\x0b\x32*.datafusion.PhysicalSortExprNodeCollectionJ\x04\x08\n\x10\x0b\"}\n\x13ParquetScanExecNode\x12/\n\tbase_conf\x18\x01 \x01(\x0b\x32\x1c.datafusion.FileScanExecConf\x12/\n\tpredicate\x18\x03 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNodeJ\x04\x08\x02\x10\x03\"i\n\x0f\x43svScanExecNode\x12/\n\tbase_conf\x18\x01 \x01(\x0b\x32\x1c.datafusion.FileScanExecConf\x12\x12\n\nhas_header\x18\x02 \x01(\x08\x12\x11\n\tdelimiter\x18\x03 \x01(\t\"C\n\x10\x41vroScanExecNode\x12/\n\tbase_conf\x18\x01 \x01(\x0b\x32\x1c.datafusion.FileScanExecConf\"\xa9\x02\n\x10HashJoinExecNode\x12*\n\x04left\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalPlanNode\x12+\n\x05right\x18\x02 \x01(\x0b\x32\x1c.datafusion.PhysicalPlanNode\x12\x1e\n\x02on\x18\x03 \x03(\x0b\x32\x12.datafusion.JoinOn\x12\'\n\tjoin_type\x18\x04 \x01(\x0e\x32\x14.datafusion.JoinType\x12\x31\n\x0epartition_mode\x18\x06 \x01(\x0e\x32\x19.datafusion.PartitionMode\x12\x18\n\x10null_equals_null\x18\x07 \x01(\x08\x12&\n\x06\x66ilter\x18\x08 \x01(\x0b\x32\x16.datafusion.JoinFilter\"=\n\rUnionExecNode\x12,\n\x06inputs\x18\x01 \x03(\x0b\x32\x1c.datafusion.PhysicalPlanNode\"~\n\x0f\x45xplainExecNode\x12\"\n\x06schema\x18\x01 \x01(\x0b\x32\x12.datafusion.Schema\x12\x36\n\x11stringified_plans\x18\x02 \x03(\x0b\x32\x1b.datafusion.StringifiedPlan\x12\x0f\n\x07verbose\x18\x03 \x01(\x08\"l\n\x11\x43rossJoinExecNode\x12*\n\x04left\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalPlanNode\x12+\n\x05right\x18\x02 \x01(\x0b\x32\x1c.datafusion.PhysicalPlanNode\"-\n\x0ePhysicalColumn\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05index\x18\x02 \x01(\r\"]\n\x06JoinOn\x12(\n\x04left\x18\x01 \x01(\x0b\x32\x1a.datafusion.PhysicalColumn\x12)\n\x05right\x18\x02 \x01(\x0b\x32\x1a.datafusion.PhysicalColumn\"L\n\rEmptyExecNode\x12\x17\n\x0fproduce_one_row\x18\x01 \x01(\x08\x12\"\n\x06schema\x18\x02 \x01(\x0b\x32\x12.datafusion.Schema\"\x80\x01\n\x12ProjectionExecNode\x12+\n\x05input\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalPlanNode\x12*\n\x04\x65xpr\x18\x02 \x03(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12\x11\n\texpr_name\x18\x03 \x03(\t\"\xb7\x01\n\x11WindowAggExecNode\x12+\n\x05input\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalPlanNode\x12\x31\n\x0bwindow_expr\x18\x02 \x03(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12\x18\n\x10window_expr_name\x18\x03 \x03(\t\x12(\n\x0cinput_schema\x18\x04 \x01(\x0b\x32\x12.datafusion.Schema\"9\n\x0bMaybeFilter\x12*\n\x04\x65xpr\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\"M\n\x16MaybePhysicalSortExprs\x12\x33\n\tsort_expr\x18\x01 \x03(\x0b\x32 .datafusion.PhysicalSortExprNode\"\xd1\x03\n\x11\x41ggregateExecNode\x12\x30\n\ngroup_expr\x18\x01 \x03(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12/\n\taggr_expr\x18\x02 \x03(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12\'\n\x04mode\x18\x03 \x01(\x0e\x32\x19.datafusion.AggregateMode\x12+\n\x05input\x18\x04 \x01(\x0b\x32\x1c.datafusion.PhysicalPlanNode\x12\x17\n\x0fgroup_expr_name\x18\x05 \x03(\t\x12\x16\n\x0e\x61ggr_expr_name\x18\x06 \x03(\t\x12(\n\x0cinput_schema\x18\x07 \x01(\x0b\x32\x12.datafusion.Schema\x12/\n\tnull_expr\x18\x08 \x03(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12\x0e\n\x06groups\x18\t \x03(\x08\x12,\n\x0b\x66ilter_expr\x18\n \x03(\x0b\x32\x17.datafusion.MaybeFilter\x12\x39\n\rorder_by_expr\x18\x0b \x03(\x0b\x32\".datafusion.MaybePhysicalSortExprs\"_\n\x13GlobalLimitExecNode\x12+\n\x05input\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalPlanNode\x12\x0c\n\x04skip\x18\x02 \x01(\r\x12\r\n\x05\x66\x65tch\x18\x03 \x01(\x03\"P\n\x12LocalLimitExecNode\x12+\n\x05input\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalPlanNode\x12\r\n\x05\x66\x65tch\x18\x02 \x01(\r\"\x95\x01\n\x0cSortExecNode\x12+\n\x05input\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalPlanNode\x12*\n\x04\x65xpr\x18\x02 \x03(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12\r\n\x05\x66\x65tch\x18\x03 \x01(\x03\x12\x1d\n\x15preserve_partitioning\x18\x04 \x01(\x08\"\x85\x01\n\x1bSortPreservingMergeExecNode\x12+\n\x05input\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalPlanNode\x12*\n\x04\x65xpr\x18\x02 \x03(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12\r\n\x05\x66\x65tch\x18\x03 \x01(\x03\"\xc2\x01\n\x16NestedLoopJoinExecNode\x12*\n\x04left\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalPlanNode\x12+\n\x05right\x18\x02 \x01(\x0b\x32\x1c.datafusion.PhysicalPlanNode\x12\'\n\tjoin_type\x18\x03 \x01(\x0e\x32\x14.datafusion.JoinType\x12&\n\x06\x66ilter\x18\x04 \x01(\x0b\x32\x16.datafusion.JoinFilter\"a\n\x17\x43oalesceBatchesExecNode\x12+\n\x05input\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalPlanNode\x12\x19\n\x11target_batch_size\x18\x02 \x01(\r\"I\n\x1a\x43oalescePartitionsExecNode\x12+\n\x05input\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalPlanNode\"c\n\x17PhysicalHashRepartition\x12/\n\thash_expr\x18\x01 \x03(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12\x17\n\x0fpartition_count\x18\x02 \x01(\x04\"\xb5\x01\n\x13RepartitionExecNode\x12+\n\x05input\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalPlanNode\x12\x15\n\x0bround_robin\x18\x02 \x01(\x04H\x00\x12\x33\n\x04hash\x18\x03 \x01(\x0b\x32#.datafusion.PhysicalHashRepartitionH\x00\x12\x11\n\x07unknown\x18\x04 \x01(\x04H\x00\x42\x12\n\x10partition_method\"\x93\x01\n\nJoinFilter\x12\x30\n\nexpression\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12/\n\x0e\x63olumn_indices\x18\x02 \x03(\x0b\x32\x17.datafusion.ColumnIndex\x12\"\n\x06schema\x18\x03 \x01(\x0b\x32\x12.datafusion.Schema\"@\n\x0b\x43olumnIndex\x12\r\n\x05index\x18\x01 \x01(\r\x12\"\n\x04side\x18\x02 \x01(\x0e\x32\x14.datafusion.JoinSide\"\xa0\x01\n\x0fPartitionedFile\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x0c\n\x04size\x18\x02 \x01(\x04\x12\x18\n\x10last_modified_ns\x18\x03 \x01(\x04\x12\x31\n\x10partition_values\x18\x04 \x03(\x0b\x32\x17.datafusion.ScalarValue\x12$\n\x05range\x18\x05 \x01(\x0b\x32\x15.datafusion.FileRange\"\'\n\tFileRange\x12\r\n\x05start\x18\x01 \x01(\x03\x12\x0b\n\x03\x65nd\x18\x02 \x01(\x03\"y\n\x0ePartitionStats\x12\x10\n\x08num_rows\x18\x01 \x01(\x03\x12\x13\n\x0bnum_batches\x18\x02 \x01(\x03\x12\x11\n\tnum_bytes\x18\x03 \x01(\x03\x12-\n\x0c\x63olumn_stats\x18\x04 \x03(\x0b\x32\x17.datafusion.ColumnStats\"x\n\nStatistics\x12\x10\n\x08num_rows\x18\x01 \x01(\x03\x12\x17\n\x0ftotal_byte_size\x18\x02 \x01(\x03\x12-\n\x0c\x63olumn_stats\x18\x03 \x03(\x0b\x32\x17.datafusion.ColumnStats\x12\x10\n\x08is_exact\x18\x04 \x01(\x08\"\x91\x01\n\x0b\x43olumnStats\x12*\n\tmin_value\x18\x01 \x01(\x0b\x32\x17.datafusion.ScalarValue\x12*\n\tmax_value\x18\x02 \x01(\x0b\x32\x17.datafusion.ScalarValue\x12\x12\n\nnull_count\x18\x03 \x01(\r\x12\x16\n\x0e\x64istinct_count\x18\x04 \x01(\r\"r\n\x1fPhysicalGetIndexedFieldExprNode\x12)\n\x03\x61rg\x18\x01 \x01(\x0b\x32\x1c.datafusion.PhysicalExprNode\x12$\n\x03key\x18\x02 \x01(\x0b\x32\x17.datafusion.ScalarValue*n\n\x08JoinType\x12\t\n\x05INNER\x10\x00\x12\x08\n\x04LEFT\x10\x01\x12\t\n\x05RIGHT\x10\x02\x12\x08\n\x04\x46ULL\x10\x03\x12\x0c\n\x08LEFTSEMI\x10\x04\x12\x0c\n\x08LEFTANTI\x10\x05\x12\r\n\tRIGHTSEMI\x10\x06\x12\r\n\tRIGHTANTI\x10\x07*#\n\x0eJoinConstraint\x12\x06\n\x02ON\x10\x00\x12\t\n\x05USING\x10\x01*\xe6\n\n\x0eScalarFunction\x12\x07\n\x03\x41\x62s\x10\x00\x12\x08\n\x04\x41\x63os\x10\x01\x12\x08\n\x04\x41sin\x10\x02\x12\x08\n\x04\x41tan\x10\x03\x12\t\n\x05\x41scii\x10\x04\x12\x08\n\x04\x43\x65il\x10\x05\x12\x07\n\x03\x43os\x10\x06\x12\n\n\x06\x44igest\x10\x07\x12\x07\n\x03\x45xp\x10\x08\x12\t\n\x05\x46loor\x10\t\x12\x06\n\x02Ln\x10\n\x12\x07\n\x03Log\x10\x0b\x12\t\n\x05Log10\x10\x0c\x12\x08\n\x04Log2\x10\r\x12\t\n\x05Round\x10\x0e\x12\n\n\x06Signum\x10\x0f\x12\x07\n\x03Sin\x10\x10\x12\x08\n\x04Sqrt\x10\x11\x12\x07\n\x03Tan\x10\x12\x12\t\n\x05Trunc\x10\x13\x12\t\n\x05\x41rray\x10\x14\x12\x0f\n\x0bRegexpMatch\x10\x15\x12\r\n\tBitLength\x10\x16\x12\t\n\x05\x42trim\x10\x17\x12\x13\n\x0f\x43haracterLength\x10\x18\x12\x07\n\x03\x43hr\x10\x19\x12\n\n\x06\x43oncat\x10\x1a\x12\x17\n\x13\x43oncatWithSeparator\x10\x1b\x12\x0c\n\x08\x44\x61tePart\x10\x1c\x12\r\n\tDateTrunc\x10\x1d\x12\x0b\n\x07InitCap\x10\x1e\x12\x08\n\x04Left\x10\x1f\x12\x08\n\x04Lpad\x10 \x12\t\n\x05Lower\x10!\x12\t\n\x05Ltrim\x10\"\x12\x07\n\x03MD5\x10#\x12\n\n\x06NullIf\x10$\x12\x0f\n\x0bOctetLength\x10%\x12\n\n\x06Random\x10&\x12\x11\n\rRegexpReplace\x10\'\x12\n\n\x06Repeat\x10(\x12\x0b\n\x07Replace\x10)\x12\x0b\n\x07Reverse\x10*\x12\t\n\x05Right\x10+\x12\x08\n\x04Rpad\x10,\x12\t\n\x05Rtrim\x10-\x12\n\n\x06SHA224\x10.\x12\n\n\x06SHA256\x10/\x12\n\n\x06SHA384\x10\x30\x12\n\n\x06SHA512\x10\x31\x12\r\n\tSplitPart\x10\x32\x12\x0e\n\nStartsWith\x10\x33\x12\n\n\x06Strpos\x10\x34\x12\n\n\x06Substr\x10\x35\x12\t\n\x05ToHex\x10\x36\x12\x0f\n\x0bToTimestamp\x10\x37\x12\x15\n\x11ToTimestampMillis\x10\x38\x12\x15\n\x11ToTimestampMicros\x10\x39\x12\x16\n\x12ToTimestampSeconds\x10:\x12\x07\n\x03Now\x10;\x12\r\n\tTranslate\x10<\x12\x08\n\x04Trim\x10=\x12\t\n\x05Upper\x10>\x12\x0c\n\x08\x43oalesce\x10?\x12\t\n\x05Power\x10@\x12\r\n\tStructFun\x10\x41\x12\x10\n\x0c\x46romUnixtime\x10\x42\x12\t\n\x05\x41tan2\x10\x43\x12\x0b\n\x07\x44\x61teBin\x10\x44\x12\x0f\n\x0b\x41rrowTypeof\x10\x45\x12\x0f\n\x0b\x43urrentDate\x10\x46\x12\x0f\n\x0b\x43urrentTime\x10G\x12\x08\n\x04Uuid\x10H\x12\x08\n\x04\x43\x62rt\x10I\x12\t\n\x05\x41\x63osh\x10J\x12\t\n\x05\x41sinh\x10K\x12\t\n\x05\x41tanh\x10L\x12\x08\n\x04Sinh\x10M\x12\x08\n\x04\x43osh\x10N\x12\x08\n\x04Tanh\x10O\x12\x06\n\x02Pi\x10P\x12\x0b\n\x07\x44\x65grees\x10Q\x12\x0b\n\x07Radians\x10R\x12\r\n\tFactorial\x10S\x12\x07\n\x03Lcm\x10T\x12\x07\n\x03Gcd\x10U\x12\x0f\n\x0b\x41rrayAppend\x10V\x12\x0f\n\x0b\x41rrayConcat\x10W\x12\r\n\tArrayDims\x10X\x12\r\n\tArrayFill\x10Y\x12\x0f\n\x0b\x41rrayLength\x10Z\x12\x0e\n\nArrayNdims\x10[\x12\x11\n\rArrayPosition\x10\\\x12\x12\n\x0e\x41rrayPositions\x10]\x12\x10\n\x0c\x41rrayPrepend\x10^\x12\x0f\n\x0b\x41rrayRemove\x10_\x12\x10\n\x0c\x41rrayReplace\x10`\x12\x11\n\rArrayToString\x10\x61\x12\x0f\n\x0b\x43\x61rdinality\x10\x62\x12\r\n\tTrimArray\x10\x63\x12\x11\n\rArrayContains\x10\x64\x12\n\n\x06\x45ncode\x10\x65\x12\n\n\x06\x44\x65\x63ode\x10\x66\x12\x07\n\x03\x43ot\x10g*\xb2\x03\n\x11\x41ggregateFunction\x12\x07\n\x03MIN\x10\x00\x12\x07\n\x03MAX\x10\x01\x12\x07\n\x03SUM\x10\x02\x12\x07\n\x03\x41VG\x10\x03\x12\t\n\x05\x43OUNT\x10\x04\x12\x13\n\x0f\x41PPROX_DISTINCT\x10\x05\x12\r\n\tARRAY_AGG\x10\x06\x12\x0c\n\x08VARIANCE\x10\x07\x12\x10\n\x0cVARIANCE_POP\x10\x08\x12\x0e\n\nCOVARIANCE\x10\t\x12\x12\n\x0e\x43OVARIANCE_POP\x10\n\x12\n\n\x06STDDEV\x10\x0b\x12\x0e\n\nSTDDEV_POP\x10\x0c\x12\x0f\n\x0b\x43ORRELATION\x10\r\x12\x1a\n\x16\x41PPROX_PERCENTILE_CONT\x10\x0e\x12\x11\n\rAPPROX_MEDIAN\x10\x0f\x12&\n\"APPROX_PERCENTILE_CONT_WITH_WEIGHT\x10\x10\x12\x0c\n\x08GROUPING\x10\x11\x12\n\n\x06MEDIAN\x10\x12\x12\x0b\n\x07\x42IT_AND\x10\x13\x12\n\n\x06\x42IT_OR\x10\x14\x12\x0b\n\x07\x42IT_XOR\x10\x15\x12\x0c\n\x08\x42OOL_AND\x10\x16\x12\x0b\n\x07\x42OOL_OR\x10\x17\x12\x13\n\x0f\x46IRST_VALUE_AGG\x10\x18\x12\x12\n\x0eLAST_VALUE_AGG\x10\x19*\xb0\x01\n\x15\x42uiltInWindowFunction\x12\x0e\n\nROW_NUMBER\x10\x00\x12\x08\n\x04RANK\x10\x01\x12\x0e\n\nDENSE_RANK\x10\x02\x12\x10\n\x0cPERCENT_RANK\x10\x03\x12\r\n\tCUME_DIST\x10\x04\x12\t\n\x05NTILE\x10\x05\x12\x07\n\x03LAG\x10\x06\x12\x08\n\x04LEAD\x10\x07\x12\x0f\n\x0b\x46IRST_VALUE\x10\x08\x12\x0e\n\nLAST_VALUE\x10\t\x12\r\n\tNTH_VALUE\x10\n*3\n\x10WindowFrameUnits\x12\x08\n\x04ROWS\x10\x00\x12\t\n\x05RANGE\x10\x01\x12\n\n\x06GROUPS\x10\x02*E\n\x14WindowFrameBoundType\x12\x0f\n\x0b\x43URRENT_ROW\x10\x00\x12\r\n\tPRECEDING\x10\x01\x12\r\n\tFOLLOWING\x10\x02*(\n\x08\x44\x61teUnit\x12\x07\n\x03\x44\x61y\x10\x00\x12\x13\n\x0f\x44\x61teMillisecond\x10\x01*H\n\x08TimeUnit\x12\n\n\x06Second\x10\x00\x12\x0f\n\x0bMillisecond\x10\x01\x12\x0f\n\x0bMicrosecond\x10\x02\x12\x0e\n\nNanosecond\x10\x03*<\n\x0cIntervalUnit\x12\r\n\tYearMonth\x10\x00\x12\x0b\n\x07\x44\x61yTime\x10\x01\x12\x10\n\x0cMonthDayNano\x10\x02*\"\n\tUnionMode\x12\n\n\x06sparse\x10\x00\x12\t\n\x05\x64\x65nse\x10\x01*<\n\rPartitionMode\x12\x10\n\x0c\x43OLLECT_LEFT\x10\x00\x12\x0f\n\x0bPARTITIONED\x10\x01\x12\x08\n\x04\x41UTO\x10\x02*b\n\rAggregateMode\x12\x0b\n\x07PARTIAL\x10\x00\x12\t\n\x05\x46INAL\x10\x01\x12\x15\n\x11\x46INAL_PARTITIONED\x10\x02\x12\n\n\x06SINGLE\x10\x03\x12\x16\n\x12SINGLE_PARTITIONED\x10\x04*)\n\x08JoinSide\x12\r\n\tLEFT_SIDE\x10\x00\x12\x0e\n\nRIGHT_SIDE\x10\x01\x42\x39\n$org.apache.arrow.datafusion.protobufB\x0f\x44\x61tafusionProtoP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'proto.datafusion_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n$org.apache.arrow.datafusion.protobufB\017DatafusionProtoP\001'
  _DFSCHEMA_METADATAENTRY._options = None
  _DFSCHEMA_METADATAENTRY._serialized_options = b'8\001'
  _CREATEEXTERNALTABLENODE_OPTIONSENTRY._options = None
  _CREATEEXTERNALTABLENODE_OPTIONSENTRY._serialized_options = b'8\001'
  _SCHEMA_METADATAENTRY._options = None
  _SCHEMA_METADATAENTRY._serialized_options = b'8\001'
  _FIELD_METADATAENTRY._options = None
  _FIELD_METADATAENTRY._serialized_options = b'8\001'
  _globals['_JOINTYPE']._serialized_start=26361
  _globals['_JOINTYPE']._serialized_end=26471
  _globals['_JOINCONSTRAINT']._serialized_start=26473
  _globals['_JOINCONSTRAINT']._serialized_end=26508
  _globals['_SCALARFUNCTION']._serialized_start=26511
  _globals['_SCALARFUNCTION']._serialized_end=27893
  _globals['_AGGREGATEFUNCTION']._serialized_start=27896
  _globals['_AGGREGATEFUNCTION']._serialized_end=28330
  _globals['_BUILTINWINDOWFUNCTION']._serialized_start=28333
  _globals['_BUILTINWINDOWFUNCTION']._serialized_end=28509
  _globals['_WINDOWFRAMEUNITS']._serialized_start=28511
  _globals['_WINDOWFRAMEUNITS']._serialized_end=28562
  _globals['_WINDOWFRAMEBOUNDTYPE']._serialized_start=28564
  _globals['_WINDOWFRAMEBOUNDTYPE']._serialized_end=28633
  _globals['_DATEUNIT']._serialized_start=28635
  _globals['_DATEUNIT']._serialized_end=28675
  _globals['_TIMEUNIT']._serialized_start=28677
  _globals['_TIMEUNIT']._serialized_end=28749
  _globals['_INTERVALUNIT']._serialized_start=28751
  _globals['_INTERVALUNIT']._serialized_end=28811
  _globals['_UNIONMODE']._serialized_start=28813
  _globals['_UNIONMODE']._serialized_end=28847
  _globals['_PARTITIONMODE']._serialized_start=28849
  _globals['_PARTITIONMODE']._serialized_end=28909
  _globals['_AGGREGATEMODE']._serialized_start=28911
  _globals['_AGGREGATEMODE']._serialized_end=29009
  _globals['_JOINSIDE']._serialized_start=29011
  _globals['_JOINSIDE']._serialized_end=29052
  _globals['_COLUMNRELATION']._serialized_start=38
  _globals['_COLUMNRELATION']._serialized_end=72
  _globals['_COLUMN']._serialized_start=74
  _globals['_COLUMN']._serialized_end=142
  _globals['_DFFIELD']._serialized_start=144
  _globals['_DFFIELD']._serialized_end=234
  _globals['_DFSCHEMA']._serialized_start=237
  _globals['_DFSCHEMA']._serialized_end=388
  _globals['_DFSCHEMA_METADATAENTRY']._serialized_start=341
  _globals['_DFSCHEMA_METADATAENTRY']._serialized_end=388
  _globals['_LOGICALPLANNODE']._serialized_start=391
  _globals['_LOGICALPLANNODE']._serialized_end=1722
  _globals['_LOGICALEXTENSIONNODE']._serialized_start=1724
  _globals['_LOGICALEXTENSIONNODE']._serialized_end=1805
  _globals['_PROJECTIONCOLUMNS']._serialized_start=1807
  _globals['_PROJECTIONCOLUMNS']._serialized_end=1843
  _globals['_CSVFORMAT']._serialized_start=1845
  _globals['_CSVFORMAT']._serialized_end=1895
  _globals['_PARQUETFORMAT']._serialized_start=1897
  _globals['_PARQUETFORMAT']._serialized_end=1918
  _globals['_AVROFORMAT']._serialized_start=1920
  _globals['_AVROFORMAT']._serialized_end=1932
  _globals['_LOGICALEXPRNODECOLLECTION']._serialized_start=1934
  _globals['_LOGICALEXPRNODECOLLECTION']._serialized_end=2018
  _globals['_LISTINGTABLESCANNODE']._serialized_start=2021
  _globals['_LISTINGTABLESCANNODE']._serialized_end=2559
  _globals['_VIEWTABLESCANNODE']._serialized_start=2562
  _globals['_VIEWTABLESCANNODE']._serialized_end=2791
  _globals['_CUSTOMTABLESCANNODE']._serialized_start=2794
  _globals['_CUSTOMTABLESCANNODE']._serialized_end=3034
  _globals['_PROJECTIONNODE']._serialized_start=3037
  _globals['_PROJECTIONNODE']._serialized_end=3175
  _globals['_SELECTIONNODE']._serialized_start=3177
  _globals['_SELECTIONNODE']._serialized_end=3279
  _globals['_SORTNODE']._serialized_start=3281
  _globals['_SORTNODE']._serialized_end=3393
  _globals['_REPARTITIONNODE']._serialized_start=3396
  _globals['_REPARTITIONNODE']._serialized_end=3545
  _globals['_HASHREPARTITION']._serialized_start=3547
  _globals['_HASHREPARTITION']._serialized_end=3637
  _globals['_EMPTYRELATIONNODE']._serialized_start=3639
  _globals['_EMPTYRELATIONNODE']._serialized_end=3683
  _globals['_CREATEEXTERNALTABLENODE']._serialized_start=3686
  _globals['_CREATEEXTERNALTABLENODE']._serialized_end=4176
  _globals['_CREATEEXTERNALTABLENODE_OPTIONSENTRY']._serialized_start=4124
  _globals['_CREATEEXTERNALTABLENODE_OPTIONSENTRY']._serialized_end=4170
  _globals['_PREPARENODE']._serialized_start=4178
  _globals['_PREPARENODE']._serialized_end=4292
  _globals['_CREATECATALOGSCHEMANODE']._serialized_start=4294
  _globals['_CREATECATALOGSCHEMANODE']._serialized_end=4401
  _globals['_CREATECATALOGNODE']._serialized_start=4403
  _globals['_CREATECATALOGNODE']._serialized_end=4505
  _globals['_DROPVIEWNODE']._serialized_start=4507
  _globals['_DROPVIEWNODE']._serialized_end=4625
  _globals['_CREATEVIEWNODE']._serialized_start=4628
  _globals['_CREATEVIEWNODE']._serialized_end=4781
  _globals['_VALUESNODE']._serialized_start=4783
  _globals['_VALUESNODE']._serialized_end=4861
  _globals['_ANALYZENODE']._serialized_start=4863
  _globals['_ANALYZENODE']._serialized_end=4937
  _globals['_EXPLAINNODE']._serialized_start=4939
  _globals['_EXPLAINNODE']._serialized_end=5013
  _globals['_AGGREGATENODE']._serialized_start=5016
  _globals['_AGGREGATENODE']._serialized_end=5172
  _globals['_WINDOWNODE']._serialized_start=5174
  _globals['_WINDOWNODE']._serialized_end=5280
  _globals['_JOINNODE']._serialized_start=5283
  _globals['_JOINNODE']._serialized_end=5650
  _globals['_DISTINCTNODE']._serialized_start=5652
  _globals['_DISTINCTNODE']._serialized_end=5710
  _globals['_UNIONNODE']._serialized_start=5712
  _globals['_UNIONNODE']._serialized_end=5768
  _globals['_CROSSJOINNODE']._serialized_start=5770
  _globals['_CROSSJOINNODE']._serialized_end=5872
  _globals['_LIMITNODE']._serialized_start=5874
  _globals['_LIMITNODE']._serialized_end=5958
  _globals['_SELECTIONEXECNODE']._serialized_start=5960
  _globals['_SELECTIONEXECNODE']._serialized_end=6022
  _globals['_SUBQUERYALIASNODE']._serialized_start=6024
  _globals['_SUBQUERYALIASNODE']._serialized_end=6141
  _globals['_LOGICALEXPRNODE']._serialized_start=6144
  _globals['_LOGICALEXPRNODE']._serialized_end=7725
  _globals['_PLACEHOLDERNODE']._serialized_start=7727
  _globals['_PLACEHOLDERNODE']._serialized_end=7798
  _globals['_LOGICALEXPRLIST']._serialized_start=7800
  _globals['_LOGICALEXPRLIST']._serialized_end=7860
  _globals['_GROUPINGSETNODE']._serialized_start=7862
  _globals['_GROUPINGSETNODE']._serialized_end=7922
  _globals['_CUBENODE']._serialized_start=7924
  _globals['_CUBENODE']._serialized_end=7977
  _globals['_ROLLUPNODE']._serialized_start=7979
  _globals['_ROLLUPNODE']._serialized_end=8034
  _globals['_GETINDEXEDFIELD']._serialized_start=8036
  _globals['_GETINDEXEDFIELD']._serialized_end=8134
  _globals['_ISNULL']._serialized_start=8136
  _globals['_ISNULL']._serialized_end=8187
  _globals['_ISNOTNULL']._serialized_start=8189
  _globals['_ISNOTNULL']._serialized_end=8243
  _globals['_ISTRUE']._serialized_start=8245
  _globals['_ISTRUE']._serialized_end=8296
  _globals['_ISFALSE']._serialized_start=8298
  _globals['_ISFALSE']._serialized_end=8350
  _globals['_ISUNKNOWN']._serialized_start=8352
  _globals['_ISUNKNOWN']._serialized_end=8406
  _globals['_ISNOTTRUE']._serialized_start=8408
  _globals['_ISNOTTRUE']._serialized_end=8462
  _globals['_ISNOTFALSE']._serialized_start=8464
  _globals['_ISNOTFALSE']._serialized_end=8519
  _globals['_ISNOTUNKNOWN']._serialized_start=8521
  _globals['_ISNOTUNKNOWN']._serialized_end=8578
  _globals['_NOT']._serialized_start=8580
  _globals['_NOT']._serialized_end=8628
  _globals['_ALIASNODE']._serialized_start=8630
  _globals['_ALIASNODE']._serialized_end=8699
  _globals['_BINARYEXPRNODE']._serialized_start=8701
  _globals['_BINARYEXPRNODE']._serialized_end=8776
  _globals['_NEGATIVENODE']._serialized_start=8778
  _globals['_NEGATIVENODE']._serialized_end=8835
  _globals['_INLISTNODE']._serialized_start=8837
  _globals['_INLISTNODE']._serialized_end=8952
  _globals['_SCALARFUNCTIONNODE']._serialized_start=8954
  _globals['_SCALARFUNCTIONNODE']._serialized_end=9058
  _globals['_AGGREGATEEXPRNODE']._serialized_start=9061
  _globals['_AGGREGATEEXPRNODE']._serialized_end=9287
  _globals['_AGGREGATEUDFEXPRNODE']._serialized_start=9290
  _globals['_AGGREGATEUDFEXPRNODE']._serialized_end=9465
  _globals['_SCALARUDFEXPRNODE']._serialized_start=9467
  _globals['_SCALARUDFEXPRNODE']._serialized_end=9547
  _globals['_WINDOWEXPRNODE']._serialized_start=9550
  _globals['_WINDOWEXPRNODE']._serialized_end=9925
  _globals['_BETWEENNODE']._serialized_start=9928
  _globals['_BETWEENNODE']._serialized_end=10086
  _globals['_LIKENODE']._serialized_start=10089
  _globals['_LIKENODE']._serialized_end=10226
  _globals['_ILIKENODE']._serialized_start=10229
  _globals['_ILIKENODE']._serialized_end=10367
  _globals['_SIMILARTONODE']._serialized_start=10370
  _globals['_SIMILARTONODE']._serialized_end=10512
  _globals['_CASENODE']._serialized_start=10515
  _globals['_CASENODE']._serialized_end=10662
  _globals['_WHENTHEN']._serialized_start=10664
  _globals['_WHENTHEN']._serialized_end=10770
  _globals['_CASTNODE']._serialized_start=10772
  _globals['_CASTNODE']._serialized_end=10868
  _globals['_TRYCASTNODE']._serialized_start=10870
  _globals['_TRYCASTNODE']._serialized_end=10969
  _globals['_SORTEXPRNODE']._serialized_start=10971
  _globals['_SORTEXPRNODE']._serialized_end=11062
  _globals['_WINDOWFRAME']._serialized_start=11065
  _globals['_WINDOWFRAME']._serialized_end=11247
  _globals['_WINDOWFRAMEBOUND']._serialized_start=11250
  _globals['_WINDOWFRAMEBOUND']._serialized_end=11381
  _globals['_SCHEMA']._serialized_start=11384
  _globals['_SCHEMA']._serialized_end=11529
  _globals['_SCHEMA_METADATAENTRY']._serialized_start=341
  _globals['_SCHEMA_METADATAENTRY']._serialized_end=388
  _globals['_FIELD']._serialized_start=11532
  _globals['_FIELD']._serialized_end=11751
  _globals['_FIELD_METADATAENTRY']._serialized_start=341
  _globals['_FIELD_METADATAENTRY']._serialized_end=388
  _globals['_FIXEDSIZEBINARY']._serialized_start=11753
  _globals['_FIXEDSIZEBINARY']._serialized_end=11786
  _globals['_TIMESTAMP']._serialized_start=11788
  _globals['_TIMESTAMP']._serialized_end=11858
  _globals['_DECIMAL']._serialized_start=11860
  _globals['_DECIMAL']._serialized_end=11915
  _globals['_LIST']._serialized_start=11917
  _globals['_LIST']._serialized_end=11962
  _globals['_FIXEDSIZELIST']._serialized_start=11964
  _globals['_FIXEDSIZELIST']._serialized_end=12037
  _globals['_DICTIONARY']._serialized_start=12039
  _globals['_DICTIONARY']._serialized_end=12125
  _globals['_STRUCT']._serialized_start=12127
  _globals['_STRUCT']._serialized_end=12179
  _globals['_MAP']._serialized_start=12181
  _globals['_MAP']._serialized_end=12246
  _globals['_UNION']._serialized_start=12248
  _globals['_UNION']._serialized_end=12356
  _globals['_SCALARLISTVALUE']._serialized_start=12358
  _globals['_SCALARLISTVALUE']._serialized_end=12467
  _globals['_SCALARTIME32VALUE']._serialized_start=12469
  _globals['_SCALARTIME32VALUE']._serialized_end=12564
  _globals['_SCALARTIME64VALUE']._serialized_start=12566
  _globals['_SCALARTIME64VALUE']._serialized_end=12665
  _globals['_SCALARTIMESTAMPVALUE']._serialized_start=12668
  _globals['_SCALARTIMESTAMPVALUE']._serialized_end=12847
  _globals['_SCALARDICTIONARYVALUE']._serialized_start=12849
  _globals['_SCALARDICTIONARYVALUE']._serialized_end=12955
  _globals['_INTERVALMONTHDAYNANOVALUE']._serialized_start=12957
  _globals['_INTERVALMONTHDAYNANOVALUE']._serialized_end=13029
  _globals['_STRUCTVALUE']._serialized_start=13031
  _globals['_STRUCTVALUE']._serialized_end=13126
  _globals['_SCALARFIXEDSIZEBINARY']._serialized_start=13128
  _globals['_SCALARFIXEDSIZEBINARY']._serialized_end=13183
  _globals['_SCALARVALUE']._serialized_start=13186
  _globals['_SCALARVALUE']._serialized_end=14417
  _globals['_DECIMAL128']._serialized_start=14419
  _globals['_DECIMAL128']._serialized_end=14468
  _globals['_ARROWTYPE']._serialized_start=14471
  _globals['_ARROWTYPE']._serialized_end=15899
  _globals['_EMPTYMESSAGE']._serialized_start=15901
  _globals['_EMPTYMESSAGE']._serialized_end=15915
  _globals['_ANALYZEDLOGICALPLANTYPE']._serialized_start=15917
  _globals['_ANALYZEDLOGICALPLANTYPE']._serialized_end=15965
  _globals['_OPTIMIZEDLOGICALPLANTYPE']._serialized_start=15967
  _globals['_OPTIMIZEDLOGICALPLANTYPE']._serialized_end=16017
  _globals['_OPTIMIZEDPHYSICALPLANTYPE']._serialized_start=16019
  _globals['_OPTIMIZEDPHYSICALPLANTYPE']._serialized_end=16070
  _globals['_PLANTYPE']._serialized_start=16073
  _globals['_PLANTYPE']._serialized_end=16595
  _globals['_STRINGIFIEDPLAN']._serialized_start=16597
  _globals['_STRINGIFIEDPLAN']._serialized_end=16669
  _globals['_BARETABLEREFERENCE']._serialized_start=16671
  _globals['_BARETABLEREFERENCE']._serialized_end=16706
  _globals['_PARTIALTABLEREFERENCE']._serialized_start=16708
  _globals['_PARTIALTABLEREFERENCE']._serialized_end=16762
  _globals['_FULLTABLEREFERENCE']._serialized_start=16764
  _globals['_FULLTABLEREFERENCE']._serialized_end=16832
  _globals['_OWNEDTABLEREFERENCE']._serialized_start=16835
  _globals['_OWNEDTABLEREFERENCE']._serialized_end=17030
  _globals['_PHYSICALPLANNODE']._serialized_start=17033
  _globals['_PHYSICALPLANNODE']._serialized_end=18195
  _globals['_PHYSICALEXTENSIONNODE']._serialized_start=18197
  _globals['_PHYSICALEXTENSIONNODE']._serialized_end=18280
  _globals['_PHYSICALEXPRNODE']._serialized_start=18283
  _globals['_PHYSICALEXPRNODE']._serialized_end=19382
  _globals['_PHYSICALSCALARUDFNODE']._serialized_start=19384
  _globals['_PHYSICALSCALARUDFNODE']._serialized_end=19509
  _globals['_PHYSICALAGGREGATEEXPRNODE']._serialized_start=19512
  _globals['_PHYSICALAGGREGATEEXPRNODE']._serialized_end=19772
  _globals['_PHYSICALWINDOWEXPRNODE']._serialized_start=19775
  _globals['_PHYSICALWINDOWEXPRNODE']._serialized_end=19982
  _globals['_PHYSICALISNULL']._serialized_start=19984
  _globals['_PHYSICALISNULL']._serialized_end=20044
  _globals['_PHYSICALISNOTNULL']._serialized_start=20046
  _globals['_PHYSICALISNOTNULL']._serialized_end=20109
  _globals['_PHYSICALNOT']._serialized_start=20111
  _globals['_PHYSICALNOT']._serialized_end=20168
  _globals['_PHYSICALALIASNODE']._serialized_start=20170
  _globals['_PHYSICALALIASNODE']._serialized_end=20248
  _globals['_PHYSICALBINARYEXPRNODE']._serialized_start=20250
  _globals['_PHYSICALBINARYEXPRNODE']._serialized_end=20368
  _globals['_PHYSICALDATETIMEINTERVALEXPRNODE']._serialized_start=20371
  _globals['_PHYSICALDATETIMEINTERVALEXPRNODE']._serialized_end=20499
  _globals['_PHYSICALLIKEEXPRNODE']._serialized_start=20502
  _globals['_PHYSICALLIKEEXPRNODE']._serialized_end=20658
  _globals['_PHYSICALSORTEXPRNODE']._serialized_start=20660
  _globals['_PHYSICALSORTEXPRNODE']._serialized_end=20760
  _globals['_PHYSICALWHENTHEN']._serialized_start=20762
  _globals['_PHYSICALWHENTHEN']._serialized_end=20878
  _globals['_PHYSICALINLISTNODE']._serialized_start=20880
  _globals['_PHYSICALINLISTNODE']._serialized_end=21005
  _globals['_PHYSICALCASENODE']._serialized_start=21008
  _globals['_PHYSICALCASENODE']._serialized_end=21173
  _globals['_PHYSICALSCALARFUNCTIONNODE']._serialized_start=21176
  _globals['_PHYSICALSCALARFUNCTIONNODE']._serialized_end=21347
  _globals['_PHYSICALTRYCASTNODE']._serialized_start=21349
  _globals['_PHYSICALTRYCASTNODE']._serialized_end=21457
  _globals['_PHYSICALCASTNODE']._serialized_start=21459
  _globals['_PHYSICALCASTNODE']._serialized_end=21564
  _globals['_PHYSICALNEGATIVENODE']._serialized_start=21566
  _globals['_PHYSICALNEGATIVENODE']._serialized_end=21632
  _globals['_FILTEREXECNODE']._serialized_start=21634
  _globals['_FILTEREXECNODE']._serialized_end=21739
  _globals['_FILEGROUP']._serialized_start=21741
  _globals['_FILEGROUP']._serialized_end=21796
  _globals['_SCANLIMIT']._serialized_start=21798
  _globals['_SCANLIMIT']._serialized_end=21824
  _globals['_PHYSICALSORTEXPRNODECOLLECTION']._serialized_start=21826
  _globals['_PHYSICALSORTEXPRNODECOLLECTION']._serialized_end=21926
  _globals['_FILESCANEXECCONF']._serialized_start=21929
  _globals['_FILESCANEXECCONF']._serialized_end=22260
  _globals['_PARQUETSCANEXECNODE']._serialized_start=22262
  _globals['_PARQUETSCANEXECNODE']._serialized_end=22387
  _globals['_CSVSCANEXECNODE']._serialized_start=22389
  _globals['_CSVSCANEXECNODE']._serialized_end=22494
  _globals['_AVROSCANEXECNODE']._serialized_start=22496
  _globals['_AVROSCANEXECNODE']._serialized_end=22563
  _globals['_HASHJOINEXECNODE']._serialized_start=22566
  _globals['_HASHJOINEXECNODE']._serialized_end=22863
  _globals['_UNIONEXECNODE']._serialized_start=22865
  _globals['_UNIONEXECNODE']._serialized_end=22926
  _globals['_EXPLAINEXECNODE']._serialized_start=22928
  _globals['_EXPLAINEXECNODE']._serialized_end=23054
  _globals['_CROSSJOINEXECNODE']._serialized_start=23056
  _globals['_CROSSJOINEXECNODE']._serialized_end=23164
  _globals['_PHYSICALCOLUMN']._serialized_start=23166
  _globals['_PHYSICALCOLUMN']._serialized_end=23211
  _globals['_JOINON']._serialized_start=23213
  _globals['_JOINON']._serialized_end=23306
  _globals['_EMPTYEXECNODE']._serialized_start=23308
  _globals['_EMPTYEXECNODE']._serialized_end=23384
  _globals['_PROJECTIONEXECNODE']._serialized_start=23387
  _globals['_PROJECTIONEXECNODE']._serialized_end=23515
  _globals['_WINDOWAGGEXECNODE']._serialized_start=23518
  _globals['_WINDOWAGGEXECNODE']._serialized_end=23701
  _globals['_MAYBEFILTER']._serialized_start=23703
  _globals['_MAYBEFILTER']._serialized_end=23760
  _globals['_MAYBEPHYSICALSORTEXPRS']._serialized_start=23762
  _globals['_MAYBEPHYSICALSORTEXPRS']._serialized_end=23839
  _globals['_AGGREGATEEXECNODE']._serialized_start=23842
  _globals['_AGGREGATEEXECNODE']._serialized_end=24307
  _globals['_GLOBALLIMITEXECNODE']._serialized_start=24309
  _globals['_GLOBALLIMITEXECNODE']._serialized_end=24404
  _globals['_LOCALLIMITEXECNODE']._serialized_start=24406
  _globals['_LOCALLIMITEXECNODE']._serialized_end=24486
  _globals['_SORTEXECNODE']._serialized_start=24489
  _globals['_SORTEXECNODE']._serialized_end=24638
  _globals['_SORTPRESERVINGMERGEEXECNODE']._serialized_start=24641
  _globals['_SORTPRESERVINGMERGEEXECNODE']._serialized_end=24774
  _globals['_NESTEDLOOPJOINEXECNODE']._serialized_start=24777
  _globals['_NESTEDLOOPJOINEXECNODE']._serialized_end=24971
  _globals['_COALESCEBATCHESEXECNODE']._serialized_start=24973
  _globals['_COALESCEBATCHESEXECNODE']._serialized_end=25070
  _globals['_COALESCEPARTITIONSEXECNODE']._serialized_start=25072
  _globals['_COALESCEPARTITIONSEXECNODE']._serialized_end=25145
  _globals['_PHYSICALHASHREPARTITION']._serialized_start=25147
  _globals['_PHYSICALHASHREPARTITION']._serialized_end=25246
  _globals['_REPARTITIONEXECNODE']._serialized_start=25249
  _globals['_REPARTITIONEXECNODE']._serialized_end=25430
  _globals['_JOINFILTER']._serialized_start=25433
  _globals['_JOINFILTER']._serialized_end=25580
  _globals['_COLUMNINDEX']._serialized_start=25582
  _globals['_COLUMNINDEX']._serialized_end=25646
  _globals['_PARTITIONEDFILE']._serialized_start=25649
  _globals['_PARTITIONEDFILE']._serialized_end=25809
  _globals['_FILERANGE']._serialized_start=25811
  _globals['_FILERANGE']._serialized_end=25850
  _globals['_PARTITIONSTATS']._serialized_start=25852
  _globals['_PARTITIONSTATS']._serialized_end=25973
  _globals['_STATISTICS']._serialized_start=25975
  _globals['_STATISTICS']._serialized_end=26095
  _globals['_COLUMNSTATS']._serialized_start=26098
  _globals['_COLUMNSTATS']._serialized_end=26243
  _globals['_PHYSICALGETINDEXEDFIELDEXPRNODE']._serialized_start=26245
  _globals['_PHYSICALGETINDEXEDFIELDEXPRNODE']._serialized_end=26359
# @@protoc_insertion_point(module_scope)



# https://altair-viz.github.io/gallery/layered_heatmap_text.html

import altair as alt
from vega_datasets import data

source = data.cars()

# Configure common options
base = alt.Chart(source).transform_aggregate(
    num_cars='count()',
    groupby=['Origin', 'Cylinders']
).encode(
    alt.X('Cylinders:O', scale=alt.Scale(paddingInner=0)),
    alt.Y('Origin:O', scale=alt.Scale(paddingInner=0)),
)

# Configure heatmap
heatmap = base.mark_rect().encode(
    color=alt.Color('num_cars:Q',
                    scale=alt.Scale(scheme='viridis'),
                    legend=alt.Legend(direction='horizontal')
                    )
)

# Configure text
text = base.mark_text(baseline='middle').encode(
    text='num_cars:Q',
    color=alt.condition(
        alt.datum.num_cars > 100,
        alt.value('black'),
        alt.value('white')
    )
)

# Draw the chart
heatmap + text



# https://altair-viz.github.io/gallery/density_facet.html
import altair as alt
from vega_datasets import data

source = data.iris()

alt.Chart(source).transform_fold(
    ['petalWidth',
     'petalLength',
     'sepalWidth',
     'sepalLength'],
    as_ = ['Measurement_type', 'value']
).transform_density(
    density='value',
    bandwidth=0.3,
    groupby=['Measurement_type'],
    extent= [0, 8]
).mark_area().encode(
    alt.X('value:Q'),
    alt.Y('density:Q'),
    alt.Row('Measurement_type:N')
).properties(width=300, height=50)


# https://altair-viz.github.io/gallery/one_dot_per_zipcode.html
# With smaller subplot
import altair as alt
from vega_datasets import data

# Since the data is more than 5,000 rows we'll import it from a URL
source = data.zipcodes.url

alt.Chart(source).transform_calculate(
    "leading digit", alt.expr.substring(alt.datum.zip_code, 0, 1)
).mark_circle(size=3).encode(
    longitude='longitude:Q',
    latitude='latitude:Q',
    color='leading digit:N',
    tooltip='zip_code:N'
).project(
    type='albersUsa'
).properties(
    width=600,
    height=400
)



# https://altair-viz.github.io/gallery/trail_marker.html
# With year column converted to a string. VegaFusion doesn't need this, but the altair case isn't handling
# an integer year column correctly.

import altair as alt
from vega_datasets import data

source = data.wheat()
source["year"] = source.year.astype(str)

alt.Chart(source).mark_trail().encode(
    x='year:T',
    y='wheat:Q',
    size='wheat:Q'
)


# https://altair-viz.github.io/gallery/top_k_items.html

import altair as alt
from vega_datasets import data

source = data.movies.url

# Top 10 movies by IMBD rating
alt.Chart(
    source,
).mark_bar().encode(
    x=alt.X('Title:N', sort='-y'),
    y=alt.Y('IMDB_Rating:Q'),
    color=alt.Color('IMDB_Rating:Q')

).transform_window(
    rank='rank(IMDB_Rating)',
    sort=[
        alt.SortField('IMDB_Rating', order='descending'),
        alt.SortField('Title', order='ascending'),
    ]
).transform_filter(
    (alt.datum.rank < 10)
)



# https://altair-viz.github.io/gallery/bar_chart_sorted.html

import altair as alt
from vega_datasets import data

source = data.barley()

alt.Chart(source).mark_bar().encode(
    x='sum(yield):Q',
    y=alt.Y('site:N', sort='-x')
)



# https://altair-viz.github.io/gallery/scatter_with_labels.html

import altair as alt
import pandas as pd

source = pd.DataFrame({
    'x': [1, 3, 5, 7, 9],
    'y': [1, 3, 5, 7, 9],
    'label': ['A', 'B', 'C', 'D', 'E']
})

points = alt.Chart(source).mark_point().encode(
    x='x:Q',
    y='y:Q'
)

text = points.mark_text(
    align='left',
    baseline='middle',
    dx=7
).encode(
    text='label'
)

points + text



# https://altair-viz.github.io/gallery/us_incomebrackets_by_state_facet.html
# With smaller subplots and 3 columns

import altair as alt
from vega_datasets import data

states = alt.topo_feature(data.us_10m.url, 'states')
source = data.income.url

alt.Chart(source).mark_geoshape().encode(
    shape='geo:G',
    color='pct:Q',
    tooltip=['name:N', 'pct:Q'],
    facet=alt.Facet('group:N', columns=3),
).transform_lookup(
    lookup='id',
    from_=alt.LookupData(data=states, key='id'),
    as_='geo'
).properties(
    width=150,
    height=80,
).project(
    type='albersUsa'
)


from abc import ABC, abstractmethod
import pyarrow as pa
from vegafusion.proto.datafusion_pb2 import LogicalExprNode

from typing import Optional, List, Literal, Union, Any


class DataFrameOperationNotSupportedError(RuntimeError):
    """
    Exception raised when a DataFrame transform method is not
    supported
    """
    pass


class DataFrameDataset(ABC):
    """
    Python interface for VegaFusion DataFrame
    """

    @abstractmethod
    def schema(self) -> pa.Schema:
        """DataFrame's pyarrow schema"""
        raise NotImplementedError()

    @abstractmethod
    def collect(self) -> pa.Table:
        """Return DataFrame's value as a pyarrow Table"""
        raise NotImplementedError()

    def fallback(self) -> bool:
        """
        Whether VegaFusion should fall back to the built-in DataFusion connection
        when DataFrame operations are encountered that are not supported

        :return: bool
        """
        return True

    def main_thread(self) -> bool:
        """
        True if the dataset must be evaluated run on the main thread.
        This blocks multithreaded parallelization, but is sometimes required

        :return: bool
        """
        return True

    def sort(
        self, exprs: List[LogicalExprNode], limit: Optional[int]
    ) -> "DataFrameDataset":
        """
        Sort and optionally limit dataset

        :param exprs: Sort expressions
        :param limit: Max number of rows to return
        :return: DataFrameDataset
        """
        raise DataFrameOperationNotSupportedError()

    def select(self, exprs: List[LogicalExprNode]) -> "DataFrameDataset":
        """
        Select columns from Dataset. Selection expressions may include column names,
        column expressions, or window expressions

        :param exprs: Selection expressions
        :return: DataFrameDataset
        """
        raise DataFrameOperationNotSupportedError()

    def aggregate(
        self, group_exprs: List[LogicalExprNode], agg_exprs: List[LogicalExprNode]
    ) -> "DataFrameDataset":
        """
        Perform dataset aggregation. Resulting dataset includes grouping
        columns and aggregate expressions

        :param group_exprs: Expressions to group by
        :param agg_exprs: Aggregate expressions
        :return: DataFrameDataset
        """
        raise DataFrameOperationNotSupportedError()

    def joinaggregate(
        self, group_exprs: List[LogicalExprNode], agg_exprs: List[LogicalExprNode]
    ) -> "DataFrameDataset":
        """
        Perform joinaggregate dataset operation.

        See: https://vega.github.io/vega/docs/transforms/joinaggregate/

        :param group_exprs: Expressions to group by
        :param agg_exprs: Aggregate expressions
        :return: DataFrameDataset
        """
        raise DataFrameOperationNotSupportedError()

    def filter(self, predicate: LogicalExprNode) -> "DataFrameDataset":
        """
        Filter dataset by predicate expression

        :param predicate: Predicate expression
        :return: DataFrameDataset
        """
        raise DataFrameOperationNotSupportedError()

    def limit(self, limit: int) -> "DataFrameDataset":
        """
        Limit dataset to max number of rows

        :param limit: Max number of rows
        :return: DataFrameDataset
        """
        raise DataFrameOperationNotSupportedError()

    def fold(
        self,
        fields: List[str],
        value_col: str,
        key_col: str,
        order_field: Optional[str],
    ) -> "DataFrameDataset":
        """
        See: https://vega.github.io/vega/docs/transforms/fold/

        :param fields: List of fields to fold
        :param value_col: Name of output value column
        :param key_col: Name of output key column
        :param order_field: Name of input ordering column or
            None if input ordering is not defined
        :return: DataFrameDataset
        """
        raise DataFrameOperationNotSupportedError()

    def stack(
        self,
        field: str,
        orderby: List[LogicalExprNode],
        groupby: List[str],
        start_field: str,
        stop_field: str,
        mode: Literal["zero", "center", "normalize"],
    ) -> "DataFrameDataset":
        """
        Computes a layout of stacking groups of values

        See: https://vega.github.io/vega/docs/transforms/stack/

        :param field: Column that determines stack height
        :param orderby: Criteria for sorting values within each stack
        :param groupby: List of columns by which to partition data into separate stacks
        :param start_field: Name of output stack start column
        :param stop_field: Name of output stack stop column
        :param mode: Stack mode. One of: "zero", "center", "normalize"
        :return:
        """
        raise DataFrameOperationNotSupportedError()

    def impute(
        self,
        field: str,
        value: Union[str, int, float],
        key: str,
        groupby: List[str],
        order_field: Optional[str],
    ) -> "DataFrameDataset":
        """
        Performs imputation of missing data objects.

        See: https://vega.github.io/vega/docs/transforms/impute/

        :param field: Column for which missing values should be imputed
        :param value: Value to impute with
        :param key: Key column that uniquely identifies data objects within a group.
            Missing key values (those occurring in the data but not in the current group)
            will be imputed.
        :param groupby: Optional list of columns to group by
        :param order_field: Optional input ordering field. If not provided, input is
            assumed to have arbitrary ordering
        :return:
        """
        raise DataFrameOperationNotSupportedError()

    def __dataframe__(
        self, nan_as_null: bool = False, allow_copy: bool = True, **kwargs
    ) -> Any:
        """DataFrame interchange protocol support"""
        table = self.collect()
        return table.__dataframe__(nan_as_null=nan_as_null, allow_copy=allow_copy)



# https://altair-viz.github.io/gallery/simple_bar_chart.html

import altair as alt
from vega_datasets import data

source = data.cars()

alt.Chart(source).mark_tick().encode(
    x='Horsepower:Q',
    y='Cylinders:O'
)



import altair as alt
import pandas as pd
import numpy as np

np.random.seed(42)

x = np.random.normal(size=100)
y = np.random.normal(size=100)

m = np.random.normal(15, 1, size=100)

source = pd.DataFrame({"x": x, "y":y, "m":m})

# interval selection in the scatter plot
pts = alt.selection_interval(encodings=["x"])

# left panel: scatter plot
points = alt.Chart().mark_point(filled=True, color="black").encode(
    x='x',
    y='y'
).transform_filter(
    pts
).properties(
    width=300,
    height=300
)

# right panel: histogram
mag = alt.Chart().mark_bar().encode(
    x='mbin:N',
    y="count()",
    color=alt.condition(pts, alt.value("black"), alt.value("lightgray"))
).properties(
    width=300,
    height=300
).add_params(pts)

# build the chart:
alt.hconcat(
    points,
    mag,
    data=source
).transform_bin(
    "mbin",
    field="m",
    bin=alt.Bin(maxbins=20)
)


# https://altair-viz.github.io/gallery/layered_chart_bar_mark.html

import altair as alt
import pandas as pd

source = pd.DataFrame({
    'project': ['a', 'b', 'c', 'd', 'e', 'f', 'g'],
    'score': [25, 57, 23, 19, 8, 47, 8],
    'goal': [25, 47, 30, 27, 38, 19, 4]
})

bar = alt.Chart(source).mark_bar().encode(
    x='project',
    y='score'
).properties(
    width=alt.Step(40)  # controls width of bar.
)

tick = alt.Chart(source).mark_tick(
    color='red',
    thickness=2,
    size=40 * 0.9,  # controls width of tick.
).encode(
    x='project',
    y='goal'
)

bar + tick


# https://altair-viz.github.io/gallery/bar_chart_with_labels.html
# Height shrunk to fit better for snapshot

import altair as alt
from vega_datasets import data

source = data.wheat()

bars = alt.Chart(source).mark_bar().encode(
    x='wheat:Q',
    y="year:O"
)

text = bars.mark_text(
    align='left',
    baseline='middle',
    dx=3  # Nudges text to right so it doesn't appear on top of the bar
).encode(
    text='wheat:Q'
)

(bars + text).properties(height=500)



from typing import Dict, Optional

import pandas as pd
import pyarrow as pa

from dataclasses import dataclass
from abc import ABC, abstractmethod


@dataclass
class CsvReadOptions:
    """
    CSV Read configuration options
    """
    has_header: bool
    delimeter: str
    file_extension: str
    schema: Optional[pa.Schema]


class RegistrationNotSupportedError(RuntimeError):
    pass


class SqlConnection(ABC):
    """
    Python interface for SQL connections
    """
    @classmethod
    def dialect(cls) -> str:
        """
        Returns SQL dialect accepted by the connection

        :return: One of:
         - "athena"
         - "bigquery"
         - "clickhouse"
         - "databricks"
         - "datafusion"
         - "duckdb"
         - "mysql"
         - "postgres"
         - "redshift"
         - "snowflake"
        """
        raise NotImplementedError()

    @abstractmethod
    def tables(self) -> Dict[str, pa.Schema]:
        """
        Returns the names and schema for the tables that are provided by the connection.
        These are the tables that may be referenced by SQL queries passed to the
        fetch_query method

        :return: dict from table name to pa.Schema
        """
        raise NotImplementedError()

    @abstractmethod
    def fetch_query(self, query: str, schema: pa.Schema) -> pa.Table:
        """
        Returns the result of evaluating the requested query. The resulting pa.Table
        should have a schema matching the provided schema

        :param query: SQL query string
        :param schema: expected pyarrow Schema of resulting pyarrow Table
        :return: pyarrow Table with query results
        """
        raise NotImplementedError()

    def fallback(self) -> bool:
        """
        Whether VegaFusion should fall back to the built-in DataFusion connection
        when SQL is encountered that is not supported by this connection's SQL dialect

        :return: bool
        """
        return True

    def register_pandas(self, name: str, df: pd.DataFrame, temporary: bool = False):
        """
        Register the provided pandas DataFrame as a table with the provided name

        :param name: Table name
        :param df: pandas DataFrame
        :param temporary: Whether table is considered temporary,
            and should be removed by unregister_temporary_tables
        """
        raise RegistrationNotSupportedError("Connection does not support registration of pandas datasets")

    def register_arrow(self, name: str, table: pa.Table, temporary: bool = False):
        """
        Register the provided pyarrow Table as a table with the provided name
        :param name: Table name
        :param table: pyarrow Table
        :param temporary: Whether table is considered temporary,
            and should be removed by unregister_temporary_tables
        """
        raise RegistrationNotSupportedError("Connection does not support registration of arrow datasets")

    def register_json(self, name: str, path: str, temporary: bool = False):
        """
        Register the JSON file at the provided path as a table with the provided name
        :param name: Table name
        :param path: Path to JSON file
        :param temporary: Whether table is considered temporary,
            and should be removed by unregister_temporary_tables
        """
        raise RegistrationNotSupportedError("Connection does not support registration of json datasets")

    def register_csv(self, name: str, path: str, options: CsvReadOptions, temporary: bool = False):
        """
        Register the CSV file at the provided path as a table with the provided name
        :param name: Table name
        :param path: Path to CSV file
        :param options: CSV options
        :param temporary: Whether table is considered temporary,
            and should be removed by unregister_temporary_tables
        """
        raise RegistrationNotSupportedError("Connection does not support registration of csv datasets")

    def register_parquet(self, name: str, path: str, temporary: bool = False):
        """
        Register the Parquet file at the provided path as a table with the provided name
        :param name: Table name
        :param path: Path to parquet file
        :param temporary: Whether table is considered temporary,
            and should be removed by unregister_temporary_tables
        """
        raise RegistrationNotSupportedError("Connection does not support registration of parquet datasets")

    def register_arrow_file(self, name: str, path: str, temporary: bool = False):
        """
        Register the Arrow file at the provided path as a table with the provided name
        :param name: Table name
        :param path: Path to arrow file (aka feather file)
        :param temporary: Whether table is considered temporary,
            and should be removed by unregister_temporary_tables
        """
        raise RegistrationNotSupportedError("Connection does not support registration of arrow file datasets")

    def unregister(self, name: str):
        """
        Unregister a table (temporary or otherwise) by name
        :param name: Table name
        """
        raise RegistrationNotSupportedError("Connection does not support unregistration")

    def unregister_temporary_tables(self):
        """
        Unregister all dynamically registered tables
        """
        raise RegistrationNotSupportedError("Connection does not support unregistering temporary tables")



# https://altair-viz.github.io/gallery/slope_graph.html

import altair as alt
from vega_datasets import data

source = data.barley()

alt.Chart(source).mark_line().encode(
    x='year:O',
    y='median(yield)',
    color='site'
)



# https://altair-viz.github.io/gallery/stacked_bar_chart_with_text.html

import altair as alt
from vega_datasets import data

source=data.barley()

bars = alt.Chart(source).mark_bar().encode(
    x=alt.X('sum(yield):Q', stack='zero'),
    y=alt.Y('variety:N'),
    color=alt.Color('site')
)

text = alt.Chart(source).mark_text(dx=-15, dy=3, color='white').encode(
    x=alt.X('sum(yield):Q', stack='zero'),
    y=alt.Y('variety:N'),
    detail='site:N',
    text=alt.Text('sum(yield):Q', format='.1f')
)

bars + text



# https://altair-viz.github.io/gallery/scatter_marginal_hist.html

import altair as alt
from vega_datasets import data

source = data.iris()

base = alt.Chart(source)

xscale = alt.Scale(domain=(4.0, 8.0))
yscale = alt.Scale(domain=(1.9, 4.55))

bar_args = {'opacity': .3, 'binSpacing': 0}

points = base.mark_circle().encode(
    alt.X('sepalLength', scale=xscale),
    alt.Y('sepalWidth', scale=yscale),
    color='species',
)

top_hist = base.mark_bar(**bar_args).encode(
    alt.X('sepalLength:Q',
          # when using bins, the axis scale is set through
          # the bin extent, so we do not specify the scale here
          # (which would be ignored anyway)
          bin=alt.Bin(maxbins=20, extent=xscale.domain),
          stack=None,
          title=''
          ),
    alt.Y('count()', stack=None, title=''),
    alt.Color('species:N'),
).properties(height=60)

right_hist = base.mark_bar(**bar_args).encode(
    alt.Y('sepalWidth:Q',
          bin=alt.Bin(maxbins=20, extent=yscale.domain),
          stack=None,
          title='',
          ),
    alt.X('count()', stack=None, title=''),
    alt.Color('species:N'),
).properties(width=60)

top_hist & (points | right_hist)



# https://altair-viz.github.io/gallery/multiple_marks.html

import altair as alt
from vega_datasets import data

source = data.stocks()

alt.Chart(source).mark_line(point=True).encode(
    x='date:T',
    y='price:Q',
    color='symbol:N'
)



from abc import ABC, abstractmethod
from typing import Iterable
import pyarrow as pa


class Datasource(ABC):
    @abstractmethod
    def schema(self) -> pa.Schema:
        raise NotImplementedError()

    @abstractmethod
    def fetch(self, columns: Iterable[str]) -> pa.Table:
        raise NotImplementedError()



# https://altair-viz.github.io/gallery/bar_and_line_with_dual_axis.html

import altair as alt
from vega_datasets import data

source = data.wheat()

base = alt.Chart(source).encode(x='year:O')

bar = base.mark_bar().encode(y='wheat:Q')

line =  base.mark_line(color='red').encode(
    y='wages:Q'
)

(bar + line).properties(width=600)



from altair.utils.plugin_registry import PluginRegistry
from typing import Callable


VegaLiteCompilerType = Callable[..., dict]


class VegaLiteCompilerRegistry(PluginRegistry[VegaLiteCompilerType]):
    pass


vegalite_compilers = VegaLiteCompilerRegistry()


def vl_convert_compiler(vegalite_spec) -> dict:
    try:
        import vl_convert as vlc
    except ImportError:
        raise ImportError(
            "The vl-convert Vega-Lite compiler requires the vl-convert-python package"
        )

    from . import altair_vl_version
    vega_spec = vlc.vegalite_to_vega(vegalite_spec, vl_version=altair_vl_version(vl_convert=True))
    return vega_spec


vegalite_compilers.register("vl-convert", vl_convert_compiler)
vegalite_compilers.enable("vl-convert")



import altair as alt
from vega_datasets import data

source = data.unemployment_across_industries.url

base = alt.Chart(source).mark_area(
    color='goldenrod',
    opacity=0.3
).encode(
    x='yearmonth(date):T',
    y='sum(count):Q',
)

brush = alt.selection_interval(encodings=['x'])
background = base.add_params(brush)
selected = base.transform_filter(brush).mark_area(color='goldenrod')

background + selected


# https://altair-viz.github.io/gallery/line_chart_with_color_datum.html

import altair as alt
from vega_datasets import data

source = data.movies()

alt.Chart(source).mark_line().encode(
    x=alt.X("IMDB_Rating", bin=True),
    y=alt.Y(
        alt.repeat("layer"), aggregate="mean", title="Mean of US and Worldwide Gross"
    ),
    color=alt.datum(alt.repeat("layer")),
).repeat(layer=["US_Gross", "Worldwide_Gross"])



from pathlib import Path
import json
import vegafusion as vf
from vl_convert import vega_to_png

from io import BytesIO
import pyarrow as pa
from skimage.io import imread
from skimage.metrics import structural_similarity as ssim
import pytest
here = Path(__file__).parent

spec_dir = here / ".." / ".." / ".." / "vegafusion-runtime" / "tests" / "specs"


def load_test_cases():
    cases = []
    for category in ["vegalite", "custom"]:
        category_dir = spec_dir / category
        for file in category_dir.glob("*.vg.json"):
            if file.name not in [
                # Undefined aggregate column
                "trellis_area_seattle.vg.json"
            ]:
                cases.append((category, file.name))

    return cases


def maybe_skip(category, name):
    if (category, name) in [
        ("vegalite", "window_cumulative_running_average.vg.json"),
        ("custom", "cumulative_running_window.vg.json"),
    ]:
        # Duckdb doesn't seem to always support multiple columns that differ only by case. e.g.
        # with top as (SELECT 1 as "year", 2 as "Year")
        #   SELECT "year", "Year" from top
        # ┌───────┬───────┐
        # │ year  │ year  │
        # │ int32 │ int32 │
        # ├───────┼───────┤
        # │     1 │     1 │
        # └───────┴───────┘
        pytest.skip("Duckdb doesn't support multiple columns that differ by case")
    elif (category, name) in [
        ("vegalite", "errorband_2d_vertical_borders.vg.json"),
        ("vegalite", "errorbar_2d_vertical_ticks.vg.json"),
        ("vegalite", "layer_line_errorband_ci.vg.json"),
        ("vegalite", "errorband_2d_horizontal_color_encoding.vg.json"),
        ("vegalite", "layer_line_errorband_2d_horizontal_borders_strokedash.vg.json"),
        ("vegalite", "layer_point_errorbar_ci.vg.json"),
        ("vegalite", "point_offset_random.vg.json"),
        ("vegalite", "sample_scatterplot.vg.json"),
        ("vegalite", "point_ordinal_bin_offset_random.vg.json"),
        ("vegalite", "layer_point_errorbar_2d_horizontal.vg.json"),
        ("vegalite", "layer_point_errorbar_2d_horizontal_ci.vg.json"),
    ]:
        # Non-deterministic (e.g. use of ci0 aggregation function)
        pytest.skip("Non-deterministic specification")


@pytest.mark.duckdb
@pytest.mark.parametrize(
    "category,name", load_test_cases())
def test_it(category, name):

    maybe_skip(category, name)

    # Load spec into dict
    file = spec_dir / category / name
    spec = json.loads(file.read_text("utf8"))

    # Define local timezone
    local_tz = "America/New_York"

    # Pre-transform with DataFusion connection and convert to image
    vf.runtime.set_connection("datafusion")
    (transformed, _) = vf.runtime.pre_transform_spec(spec, local_tz)
    img_datafusion = imread(BytesIO(vega_to_png(transformed)))

    # Pre-transform with DuckDB connection and convert to image
    vf.runtime.set_connection("duckdb")
    (transformed, _) = vf.runtime.pre_transform_spec(spec, local_tz)
    img_duckdb = imread(BytesIO(vega_to_png(transformed)))

    # Compare images
    assert img_datafusion.shape == img_duckdb.shape, "Size mismatch between datafusion and duckdb connections"
    similarity = ssim(img_datafusion, img_duckdb, channel_axis=2)
    print(similarity)
    assert similarity >= 0.998, f"Similarity failed between datafusion and duckdb connections"


def test_pretransform_extract():
    spec_file = spec_dir / "vegalite" / "rect_binned_heatmap.vg.json"
    spec = json.loads(spec_file.read_text("utf8"))

    vf.runtime.set_connection("datafusion")
    (transformed, datasets, warnings) = vf.runtime.pre_transform_extract(spec, "UTC")

    assert len(warnings) == 0
    assert len(datasets) == 1

    (name, scope, table)= datasets[0]
    assert name == "source_0"
    assert scope == []
    assert isinstance(table, pa.Table)
    assert table.shape == (379, 5)



# https://altair-viz.github.io/gallery/layered_area_chart.html

import altair as alt
from vega_datasets import data

source = data.iowa_electricity()

alt.Chart(source).mark_area(opacity=0.3).encode(
    x="year:T",
    y=alt.Y("net_generation:Q", stack=None),
    color="source:N"
)



# https://altair-viz.github.io/gallery/simple_line_chart.html

import altair as alt
import numpy as np
import pandas as pd

x = np.arange(100)
source = pd.DataFrame({
    'x': x,
    'f(x)': np.sin(x / 5)
})

alt.Chart(source).mark_line().encode(
    x='x',
    y='f(x)'
)



# https://altair-viz.github.io/gallery/errorbars_with_ci.html
# Padding updates to make size deterministic even when data is not

import altair as alt
from vega_datasets import data

source = data.barley()

error_bars = alt.Chart(source).mark_errorbar(extent='ci').encode(
    x=alt.X('yield:Q', scale=alt.Scale(zero=False)),
    y=alt.Y('variety:N')
)

points = alt.Chart(source).mark_point(filled=True, color='black').encode(
    x=alt.X('yield:Q', aggregate='mean'),
    y=alt.Y('variety:N'),
)

(error_bars + points).properties(
    padding={"left": 50, "top": 5, "right": 50, "bottom": 5}
)



# https://altair-viz.github.io/gallery/boxplot.html

import altair as alt
from vega_datasets import data

source = data.population.url

alt.Chart(source).mark_boxplot(extent='min-max').encode(
    x='age:O',
    y='people:Q'
)



__tz_config = dict(local_tz=None)


def get_local_tz():
    """
    Get the named local timezone that the VegaFusion mimetype renderer
    will use for calculations.

    Defaults to the kernel's local timezone as determined by vl-convert.

    Has no effect on VegaFusionWidget, which always uses the
    browser's local timezone

    :return: named timezone string
    """
    if __tz_config["local_tz"] is None:
        # Fall back to getting local_tz from vl-convert if not set
        try:
            import vl_convert as vlc
            __tz_config["local_tz"] = vlc.get_local_tz() or "UTC"
        except ImportError:
            raise ImportError(
                "vl-convert is not installed and so the local system timezone cannot be determined.\n"
                "Either install the vl-convert-python package or set the local timezone manually using\n"
                "the vegafusion.set_local_tz function"
            )

    return __tz_config["local_tz"]


def set_local_tz(local_tz):
    """
    Set the named local timezone that the VegaFusion mimetype renderer
    will use for calculations.

    Has no effect on VegaFusionWidget, which always uses the
    browser's local timezone

    :param local_tz: Named local timezone (e.g. "America/New_York")
    """
    __tz_config["local_tz"] = local_tz



from typing import Iterable
from math import floor
import pandas as pd
import pyarrow as pa
from .datasource import Datasource


class PandasDatasource(Datasource):
    def __init__(self, df: pd.DataFrame, sample_size: int = 1000, batch_size: int = 8096):
        fields = []
        casts = {}
        sample_stride = max(1, floor(len(df) / sample_size))

        # Shallow copy and add named index levels as columns
        # (this is faster that df.reset_index, which seems to perform a deep copy)
        df = df.copy(deep=False)
        for i, index_name in enumerate(getattr(df.index, "names", [])):
            if isinstance(index_name, str):
                df[index_name] = df.index.get_level_values(i)
        df.reset_index(drop=True, inplace=True)

        for col, pd_type in df.dtypes.items():
            if not isinstance(col, str):
                continue
            try:
                # We will expand categoricals (not yet supported in VegaFusion)
                if isinstance(pd_type, pd.CategoricalDtype):
                    cat = df[col].cat
                    field = pa.Schema.from_pandas(pd.DataFrame({col: cat.categories})).field(col)
                else:
                    candidate_schema = pa.Schema.from_pandas(df.iloc[::sample_stride][[col]])
                    field = candidate_schema.field(col)

                # Convert Decimal columns to float
                if isinstance(field.type, (pa.Decimal128Type, pa.Decimal256Type)):
                    field = pa.field(col, pa.float64())
                    casts[col] = "float64"

                # Get inferred pyarrow type
                fields.append(field)

            except pa.ArrowTypeError:
                if pd_type.kind == "O":
                    fields.append(pa.field(col, pa.string()))
                    casts[col] = str
                else:
                    raise
        self._df = df
        self._schema = pa.schema(fields)
        self._casts = casts
        self._batch_size = batch_size

    def schema(self) -> pa.Schema:
        return self._schema

    def fetch(self, columns: Iterable[str]) -> pa.Table:
        projected = self._df[columns].copy(deep=False)

        for col, pd_type in projected.dtypes.items():
            # Handle cases that need to happen before conversion to pyarrow
            if col in self._casts:
                projected[col] = projected[col].astype(self._casts[col])

            # Handle expanding categoricals
            if isinstance(pd_type, pd.CategoricalDtype):
                cat = projected[col].cat
                projected[col] = cat.categories[cat.codes]

        table = pa.Table.from_pandas(projected, preserve_index=False)

        # Split into batches of desired size
        if self._batch_size is None:
            return table
        else:
            # Resize batches
            batches = table.to_batches(self._batch_size)
            return pa.Table.from_batches(batches, table.schema)



# https://altair-viz.github.io/gallery/layer_line_color_rule.html

import altair as alt
from vega_datasets import data

source = data.stocks()

base = alt.Chart(source).properties(width=550)

line = base.mark_line().encode(
    x='date',
    y='price',
    color='symbol'
)

rule = base.mark_rule().encode(
    y='average(price)',
    color='symbol',
    size=alt.value(2)
)

line + rule



# https://altair-viz.github.io/gallery/candlestick_chart.html

import altair as alt
from vega_datasets import data

source = data.ohlc()

open_close_color = alt.condition("datum.open <= datum.close",
                                 alt.value("#06982d"),
                                 alt.value("#ae1325"))

base = alt.Chart(source).encode(
    alt.X('date:T',
          axis=alt.Axis(
              format='%m/%d',
              labelAngle=-45,
              title='Date in 2009'
          )
          ),
    color=open_close_color
)

rule = base.mark_rule().encode(
    alt.Y(
        'low:Q',
        title='Price',
        scale=alt.Scale(zero=False),
    ),
    alt.Y2('high:Q')
)

bar = base.mark_bar().encode(
    alt.Y('open:Q'),
    alt.Y2('close:Q')
)

rule + bar



# https://altair-viz.github.io/gallery/layered_chart_with_dual_axis.html

import altair as alt
from vega_datasets import data

source = data.seattle_weather()

base = alt.Chart(source).encode(
    alt.X('month(date):T', axis=alt.Axis(title=None))
)

area = base.mark_area(opacity=0.3, color='#57A44C').encode(
    alt.Y('average(temp_max)',
          axis=alt.Axis(title='Avg. Temperature (°C)', titleColor='#57A44C')),
    alt.Y2('average(temp_min)')
)

line = base.mark_line(stroke='#5276A7', interpolate='monotone').encode(
    alt.Y('average(precipitation)',
          axis=alt.Axis(title='Precipitation (inches)', titleColor='#5276A7'))
)

alt.layer(area, line).resolve_scale(
    y = 'independent'
)


# https://altair-viz.github.io/gallery/bar_chart_horizontal.html
# Height lowered to fit better

import altair as alt
from vega_datasets import data

source = data.wheat()

alt.Chart(source).mark_bar().encode(
    x='wheat:Q',
    y="year:O"
).properties(height=500)



# https://altair-viz.github.io/gallery/isotype_emoji.html

import altair as alt
import pandas as pd

source = pd.DataFrame([
    {'country': 'Great Britain', 'animal': 'cattle'},
    {'country': 'Great Britain', 'animal': 'cattle'},
    {'country': 'Great Britain', 'animal': 'cattle'},
    {'country': 'Great Britain', 'animal': 'pigs'},
    {'country': 'Great Britain', 'animal': 'pigs'},
    {'country': 'Great Britain', 'animal': 'sheep'},
    {'country': 'Great Britain', 'animal': 'sheep'},
    {'country': 'Great Britain', 'animal': 'sheep'},
    {'country': 'Great Britain', 'animal': 'sheep'},
    {'country': 'Great Britain', 'animal': 'sheep'},
    {'country': 'Great Britain', 'animal': 'sheep'},
    {'country': 'Great Britain', 'animal': 'sheep'},
    {'country': 'Great Britain', 'animal': 'sheep'},
    {'country': 'Great Britain', 'animal': 'sheep'},
    {'country': 'Great Britain', 'animal': 'sheep'},
    {'country': 'United States', 'animal': 'cattle'},
    {'country': 'United States', 'animal': 'cattle'},
    {'country': 'United States', 'animal': 'cattle'},
    {'country': 'United States', 'animal': 'cattle'},
    {'country': 'United States', 'animal': 'cattle'},
    {'country': 'United States', 'animal': 'cattle'},
    {'country': 'United States', 'animal': 'cattle'},
    {'country': 'United States', 'animal': 'cattle'},
    {'country': 'United States', 'animal': 'cattle'},
    {'country': 'United States', 'animal': 'pigs'},
    {'country': 'United States', 'animal': 'pigs'},
    {'country': 'United States', 'animal': 'pigs'},
    {'country': 'United States', 'animal': 'pigs'},
    {'country': 'United States', 'animal': 'pigs'},
    {'country': 'United States', 'animal': 'pigs'},
    {'country': 'United States', 'animal': 'sheep'},
    {'country': 'United States', 'animal': 'sheep'},
    {'country': 'United States', 'animal': 'sheep'},
    {'country': 'United States', 'animal': 'sheep'},
    {'country': 'United States', 'animal': 'sheep'},
    {'country': 'United States', 'animal': 'sheep'},
    {'country': 'United States', 'animal': 'sheep'}
])


alt.Chart(source).mark_text(size=45, baseline='middle').encode(
    alt.X('x:O', axis=None),
    alt.Y('animal:O', axis=None),
    alt.Row('country:N', header=alt.Header(title='')),
    alt.Text('emoji:N')
).transform_calculate(
    emoji="{'cattle': '🐄', 'pigs': '🐖', 'sheep': '🐏'}[datum.animal]"
).transform_window(
    x='rank()',
    groupby=['country', 'animal']
).properties(width=550, height=140)
